/*
   XAPI32MC.H - XAPI32 User functions and data structures
   Copyright (c) 1994-1996 Sangoma Technologies Inc.
   Author: Glenn Robins
   API v1.0, 32-bit, January 1995
   API v1.1, 32-bit, January 1996

   Application include file for XAPI32MC.DLL functions.
   See XAPI32.HLP for a complete function reference.

   Note that there is no difference in function prototypes between
   XAPI (for Windows 3.1) and XAPI32 at this time.  All Windows 3.1
   XAPI calls may be ported directly to Win32.

   The one significant difference is the nature of 32-bit .DLL's vs.
   16-bit .DLL's, where the XAPI32MC.DLL for example will be loaded for
   each process, and all data is unique to each process.  Multiple threads
   of a process calling the XAPI32MC.DLL will share their data (there is no
   thread local storage allocated).  The following notes on multi-threading
   are based on theory as no programs have been written to this date to
   take advantage of this capability.  In addition, the XAPI32MC.DLL and the 
   SDLA driver have not been tested on a multiprocessor platform.  A mutex
   object is used by XAPI32 to prevent multiple processes/threads from
   issuing a command to any physical card at the same time.  Another mutex
   object is used to prevent simultaneous calls to card registration-
   oriented functions.

        NOTES REGARDING MULTI-THREADING

        There are function groups whose functions must be
        called exclusively within the group as follows:
        (i.e. any function in a particular group must not be
        called simultaneously with another function (or the same
        function) in the same group, which may occur if these
        functions are used in different threads without
        synchronization.  If they are all used within the same
        one-and-only thread, there is no danger except for
        calling them in the wrong order).

        -       XAPISetup, XAPICleanup,
                XAPIRegisterDebug, XAPITerminateDebug,
                XAPIIsDebugRegistered
        
        -       XAPIControl with SET_HDLC_X25_CONFIGURATION
                command combined with ANY other LCN-oriented
                function call

                Note that the above command issued in one
                process will not update LCN-related configuration
                in other processes as it would have done in
                Windows 3.1 for different application instances

        The following functions may be called any time, from
        within any execution thread (or process), assuming they
        are called when appropriate (e.g. don't call XAPIControl
        on a non-registered card!).

        -       XAPIControl (internally mutex-controlled),
                XAPIGetCards (mutexed with card registration functions),
                XAPISupervisoryCall,
                XAPIqNotification,
                XAPIqTraceDataAvailable,
                XAPIqLinkInABM,
                XAPIqDataToRead, XAPIqClearToXmit,
                XAPIGetLCType, XAPIGetHighestLCN



   Must #include <windows.h> before this header for type
   definitions used here.

   If compiling with Borland C, make sure structures are
   packed on byte boundaries. Link your application with
   XAPI32IM.DEF.

   If compiling with MS Visual C++ and for some reason
   you don't want to use __declspec(dllimport) then define
   SANGOMAIMPORT on the compiler command line. Link a VC++
   application using XAPI32MC.LIB.
*/

#ifndef SANGOMAAPI
#define SANGOMAAPI __cdecl
#endif

#ifdef SANGOMABUILD
#define SANGOMAIMPORT
#endif

#ifndef SANGOMAIMPORT
#if (_MSC_VER >= 800) && !defined(__STDC__)
#define SANGOMAIMPORT __declspec(dllimport)
#else
#define SANGOMAIMPORT
#endif
#endif

#ifndef SANGOMABUILD
#define	SDLAR_NOT_REGISTERED	0L
#define	SDLAR_DEREGISTERED		1L
#define	SDLAR_RELEASED			2L
#define	SDLAR_REGISTERED        1L
#define	SDLAR_ALLOCATED			2L
#define	SDLAR_CARD_BLOCKED		0L
#define	SDLAR_PREPARED			2L
#define	SDLAR_NOT_PREPARED		0L
#define	SDLAR_ERROR_PREPARING	3L
#endif

#define MDATALEN        1027
#define CB_HEADERLEN    15

/* XIP control block */
#ifdef _MSC_VER
#pragma pack(1)
#endif
typedef struct {
        BYTE command;
	WORD buffer_length;
        BYTE return_code;
        BYTE reserved_HDLC;
	WORD logical_channel;
        BYTE Q_D_M_bits;
        BYTE cause;
        BYTE diagnostic;
        BYTE X25_packet_type;
        BYTE time_stamp[4];
        BYTE data[MDATALEN];
} CBLOCK;
#ifdef _MSC_VER
#pragma pack()
#endif

#if defined (__cplusplus)
extern "C" {
#endif

/* bitmasks for Q_D_M_bits above */
#define MASK_QBIT       0x04
#define MASK_DBIT       0x02
#define MASK_MBIT       0x01


/* XAPI will always return 1 */
SANGOMAIMPORT
int SANGOMAAPI XAPIAlive(void);

/* Initializes XAPI and loads SDLA driver, if not loaded already.
   A trace should be registered before this setup if one will
   be used at all, but is not necessary.
   Each application instance MUST call this on initialization.
   returns 0 if failed to load SDLA driver,
	   1 if this is the first setup call and the SDLA driver
	     loaded successfully,
	   2 if this is not the first successful setup call
	     (driver already loaded)
*/
SANGOMAIMPORT
int SANGOMAAPI XAPISetup(void);

/* Cleanup and unloads SDLA driver if this is the last
   application instance to use the driver.
   You must terminate your trace registration first if one
   was registered, unless you are the last application to use
   this DLL (but you shouldn't assume this).
   Each application instance MUST call this before closing.
   returns 1 always (return code reserved for future use)
*/
SANGOMAIMPORT
int SANGOMAAPI XAPICleanup(void);

/* returns non-zero if link in ABM */
SANGOMAIMPORT
int SANGOMAAPI XAPIqLinkInABM(BYTE cardnumber);

/* Emulate XIP interrupt call with control block
   returns return_code value in structure
*/
SANGOMAIMPORT
char SANGOMAAPI XAPIControl(CBLOCK *block, BYTE cardnumber);

/* returns non-zero if notify bit is set */
SANGOMAIMPORT
char SANGOMAAPI XAPIqNotification(BYTE cardnumber);

/* returns non-zero if trace data is available to be read by
   a READ_TRACE_DATA call
*/
SANGOMAIMPORT
char SANGOMAAPI XAPIqTraceDataAvailable(BYTE cardnumber);

/* returns non-zero if there is data to receive, or if the specified
   LC is not in data transfer mode, in which case an X25_READ should be
   issued by the application anyway to look at the return code, which
   would hopefully explain why the link is down
*/
SANGOMAIMPORT
char SANGOMAAPI XAPIqDataToRead(WORD logical_channel, BYTE cardnumber);

/* returns non-zero if there are available buffers to issue a successful
   X25_WRITE and if the transmit window is open, or if the specified LC
   is not in data transfer mode, in which case an X25_WRITE should be
   issues anyway by the application to look at the return code, which
   would hopefully explain why the link is down
*/
SANGOMAIMPORT
char SANGOMAAPI XAPIqClearToXmit(WORD logical_channel, BYTE cardnumber);

/* returns:
   0 = PVC, 1 = I-SVC (incoming only), 2 = T-SVC (two-way),
   3 = O-SVC (outgoing only), or 0xff if invalid LCN
*/
SANGOMAIMPORT
unsigned char SANGOMAAPI XAPIGetLCType(WORD lc, BYTE cardnumber);

/* returns highest LCN configured, of all SVC's or PVC's */
SANGOMAIMPORT
WORD SANGOMAAPI XAPIGetHighestLCN(BYTE cardnumber);

/* checks the board status bits for the specified information,
   without issuing a mailbox command (fast!)
*/
SANGOMAIMPORT
char SANGOMAAPI XAPISupervisoryCall(BYTE infocode, WORD lcn, BYTE cardnumber);
/* infocode as follows... */
/* returns number of packets available to be read */
#define XSC_LCINFO_NUMPACKETS   0x80
/* returns non-zero if the transmit window is open */
#define XSC_LCINFO_TXWINDOWOPEN 0x81
/* returns non-zero if the LC is in data transfer mode */
#define XSC_LCINFO_IN_DTM       0x82
#define XSC_INFO_PC             0x80    /* internal */
/* returns non-zero if there is data to read on any channel */
#define XSC_INFO_DATATOREAD     0x01
/* returns non-zero if there are buffers available to write a
   packet to the transmit queue (indep. of LC)
*/
#define XSC_INFO_BUFFSTOXMIT    0x02

/* returns 0 if a debugging command trace is already registered or
   had difficulty creating the window, 1 if successful.
   hWnd is a window to send notification messages to, and
   iMessage is a user-defined message that XAPI will send the
   application to signal an XAPI notification. The actual
   message code is passed by value to the WPARAM parameter
   of the window procedure
*/
SANGOMAIMPORT
char SANGOMAAPI XAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage);

/* if a debugging command trace is active and instance matches that passed
   when registered (for security/fault tolerance purposes) then
   trace is terminated and returns 0. If instance doesn't match,
   returns 1. If no trace is running, returns 2.
   The trace is also terminated automatically if the window is
   closed (see trace messages)
*/
SANGOMAIMPORT
char SANGOMAAPI XAPITerminateDebug(HINSTANCE hInstance);

/* returns TRUE if a debugging command trace is registered and active */
SANGOMAIMPORT
BOOL SANGOMAAPI XAPIIsDebugRegistered(void);

/* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
#define TRACEMSG_CLOSE  0

/*
   returns SDLAR_NOT_REGISTERED (0L) if failed to obtain a selector
	(any read or write calls to the board will result
	in a general protection fault)
   returns SDLAR_REGISTERED if succeeded (1L): the card's
	instance count was incremented but this is not the
	first registration call for this board (it is already
	in use by an application)
   returns SDLAR_ALLOCATED if succeeded (2L): the card's
	instance count was incremented and this is the
	first registration call for this board, and the
	memory selector was allocated
*/
SANGOMAIMPORT
int SANGOMAAPI XAPIRegisterCard(BYTE cardnumber);

/*
   returns SDLAR_NOT_REGISTERED (0L) if card wasn't registered
	or if a mutex timeout occurred
   returns SDLAR_DEREGISTERED (1L) if card's instance count
	was decremented but isn't currently zero
	(i.e. an application is still using the specified board)
   returns SDLAR_RELEASED (2L) if card is now not used by
	any application, and the selector was freed
*/
SANGOMAIMPORT
int SANGOMAAPI XAPIDeregisterCard(BYTE cardnumber);

/*
   returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
   For example, if bit R2 is set, then card 2 is registered
		if bit E2 is set, then card 2 exists
			(i.e. a configuration entry exists
			in WIN.INI)
   returns 0 if a mutex timeout occurred
*/
SANGOMAIMPORT
LRESULT SANGOMAAPI XAPIGetCards(void);

/* Undocumented DLL exports - use at own risk */
SANGOMAIMPORT int SANGOMAAPI XAPIPrepareForLoading(BYTE cardnumber);
SANGOMAIMPORT int SANGOMAAPI XAPILoadComplete(BYTE cardnumber);
SANGOMAIMPORT unsigned char SANGOMAAPI XAPIReadPortByte(BYTE cardnumber, WORD offset);
SANGOMAIMPORT void SANGOMAAPI XAPIWritePortByte(BYTE cardnumber, WORD offset, BYTE data);
SANGOMAIMPORT unsigned char SANGOMAAPI XAPIReadByte(BYTE cardnumber, WORD offset);
SANGOMAIMPORT void SANGOMAAPI XAPIWriteByte(BYTE cardnumber, WORD offset, BYTE data);
SANGOMAIMPORT void SANGOMAAPI XAPIZeroWindow(BYTE cardnumber);

#ifdef __cplusplus
}
#endif
