/* XAPI_F.H - XAPI User Functions and data structs
   Copyright (c) 1994 Sangoma Technologies Inc.
   Author: Glenn Robins
   API v0.2, 16-bit

   Must #include <windows.h> before this header
*/

#define	MDATALEN	1027
#define	CB_HEADERLEN	15

/* XIP control block */
typedef struct {
	BYTE command;
	WORD buffer_length;
	BYTE return_code;
	BYTE reserved_HDLC;
	WORD logical_channel;
	BYTE Q_D_M_bits;
	BYTE cause;
	BYTE diagnostic;
	BYTE X25_packet_type;
	BYTE time_stamp[4];
	BYTE data[MDATALEN];
} CBLOCK;

/* bitmasks for Q_D_M_bits above */
#define	MASK_QBIT	0x04
#define	MASK_DBIT	0x02
#define	MASK_MBIT	0x01


/* XAPI will always return 1 */
int WINAPI _export XAPIAlive(void);

/* Initializes XAPI and loads SDLA driver, if not loaded already.
   A trace should be registered before this setup if one will
   be used at all, but is not necessary.
   Each application instance MUST call this on initialization.
   returns 0 if failed to load SDLA driver,
	   1 if this is the first setup call and the SDLA driver
	     loaded successfully,
	   2 if this is not the first successful setup call
	     (driver already loaded)
*/
int WINAPI _export XAPISetup(void);

/* Cleanup and unloads SDLA driver if this is the last
   application instance to use the driver.
   You must terminate your trace registration first if one
   was registered, unless you are the last application to use
   this DLL (but you shouldn't assume this).
   Each application instance MUST call this before closing.
   returns 1 always (return code reserved for future use)
*/
int WINAPI _export XAPICleanup(void);

/* returns non-zero if link in ABM */
int WINAPI _export XAPIqLinkInABM(BYTE cardnumber);

/* Emulate XIP interrupt call with control block
   returns return_code value in structure
*/
char WINAPI _export XAPIControl(CBLOCK far *block, BYTE cardnumber);

/* returns non-zero if notify bit is set */
char WINAPI _export XAPIqNotification(BYTE cardnumber);

/* returns non-zero if trace data is available to be read by
   a READ_TRACE_DATA call
*/
char WINAPI _export XAPIqTraceDataAvailable(BYTE cardnumber);

/* returns non-zero if there is data to receive, or if the specified
   LCN is not in data transfer mode, in which case an X25_READ should be
   issued by the application anyway to look at the return code, which
   would hopefully explain why the link is down
*/
char WINAPI _export XAPIqDataToRead(WORD logical_channel, BYTE cardnumber);

/* returns non-zero if there are available buffers to issue a successful
   X25_WRITE and if the transmit window is open, or if the specified LCN
   is not in data transfer mode, in which case an X25_WRITE should be
   issues anyway by the application to look at the return code, which
   would hopefully explain why the link is down
*/
char WINAPI _export XAPIqClearToXmit(WORD logical_channel, BYTE cardnumber);

/* returns:
   0 = PVC, 1 = I-SVC (incoming only), 2 = T-SVC (two-way),
   3 = O-SVC (outgoing only), or 0xff if invalid LCN
*/
unsigned char WINAPI _export XAPIGetLCType(WORD lc, BYTE cardnumber);

/* returns highest LCN configured, of all SVC's or PVC's */
WORD WINAPI _export XAPIGetHighestLCN(BYTE cardnumber);

/* checks the board status bits for the specified information,
   without issuing a mailbox command (fast!)
*/
char WINAPI _export XAPISupervisoryCall(BYTE infocode, WORD lcn, BYTE cardnumber);
/* infocode as follows... */
/* returns number of packets available to be read */
#define	XSC_LCINFO_NUMPACKETS	0x80
/* returns non-zero if the transmit window is open */
#define	XSC_LCINFO_TXWINDOWOPEN	0x81
/* returns non-zero if the LC is in data transfer mode */
#define	XSC_LCINFO_IN_DTM	0x82
#define	XSC_INFO_PC		0x80	/* internal */
/* returns non-zero if there is data to read on any channel */
#define	XSC_INFO_DATATOREAD	0x01
/* returns non-zero if there are buffers available to write a
   packet to the transmit queue (indep. of LCN)
*/
#define	XSC_INFO_BUFFSTOXMIT	0x02

/* returns 0 if a debugging command trace is already registered or
   had difficulty creating the window, 1 if successful.
   hWnd is a window to send notification messages to, and
   iMessage is a user-defined message that XAPI will send the
   application to signal an XAPI notification. The actual
   message code is passed by value to the WPARAM parameter
   of the window procedure
*/
char WINAPI _export XAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage);

/* if a debugging command trace is active and instance matches that passed
   when registered (for security/fault tolerance purposes) then
   trace is terminated and returns 0. If instance doesn't match,
   returns 1. If no trace is running, returns 2.
   The trace is also terminated automatically if the window is
   closed (see trace messages)
*/
char WINAPI _export XAPITerminateDebug(HINSTANCE hInstance);

/* returns TRUE if a debugging command trace is registered and active */
BOOL WINAPI _export XAPIIsDebugRegistered(void);

/* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
#define	TRACEMSG_CLOSE	0

/*
   returns SDLAR_NOT_REGISTERED (0L) if failed to obtain a selector
	(any read or write calls to the board will result
	in a general protection fault)
   returns SDLAR_REGISTERED if succeeded (1L): the card's
	instance count was incremented but this is not the
	first registration call for this board (it is already
	in use by an application)
   returns SDLAR_ALLOCATED if succeeded (2L): the card's
	instance count was incremented and this is the
	first registration call for this board, and the
	memory selector was allocated
*/
int WINAPI _export XAPIRegisterCard(BYTE cardnumber);

/*
   returns SDLAR_NOT_REGISTERED (0L) if card wasn't registered
   returns SDLAR_DEREGISTERED (1L) if card's instance count
	was decremented but isn't currently zero
	(i.e. an application is still using the specified board)
   returns SDLAR_RELEASED (2L) if card is now not used by
	any application, and the selector was freed
*/
int WINAPI _export XAPIDeregisterCard(BYTE cardnumber);

/*
   returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
   For example, if bit R2 is set, then card 2 is registered
		if bit E2 is set, then card 2 exists
			(i.e. a configuration entry exists
			in WIN.INI)
*/
LRESULT WINAPI _export XAPIGetCards(void);

int WINAPI _export XAPIPrepareForLoading(BYTE cardnumber);
int WINAPI _export XAPILoadComplete(BYTE cardnumber);
unsigned char WINAPI _export XAPIReadPortByte(BYTE cardnumber, WORD offset);
void WINAPI _export XAPIWritePortByte(BYTE cardnumber, WORD offset, BYTE data);
unsigned char WINAPI _export XAPIReadByte(BYTE cardnumber, WORD offset);
void WINAPI _export XAPIWriteByte(BYTE cardnumber, WORD offset, BYTE data);
void WINAPI _export XAPIZeroWindow(BYTE cardnumber);

