//
// Winxldr8.C - X.25 loader for Sangoma S508 board
//
// Links to EXE
//
// Changed from Winhldr8.C by Anton on June, 1997
//
#define	STRICT

//
// Include files
//
#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include "winxldr8.h"

//
// #define's
//
#define	MSG_POLL		0x4000
#define	IDM_ABOUT		0x1000
#define	COUNT_MAX		100
#define	MAX_CARDS		16
#define	CAF				7

// Z80 commands 
#define HALT_CPU 		0x00 // halt the CPU
#define START_CPU 		0x02 // start the CPU
#define MEM_ENABLE		0x04 // enable host access to boards memory
#define NMI 			0x10 // issue an NMI to the CPU

// SDLA adapter port constants
#define OFF_PC_ADDR_WS_SELECT 	0x01	// the offset from the card I/O port base address of the adapter
										// segment and PC window select latch
#define OFF_Z80_WIN_SELECT 		0x02	// the offset from the card I/O port base address of the Z80 window
										// select latch
#define SET_8K_WINDOW 			0x01	// a bit to enable 8K window operation
#define RESET_8K_WINDOW 		0xFE	// bit 0 reset to disable 8K window operation

// Card 64K segment address settings 
// bits 3-4 of configuration byte written to port (base address + 1)
#define SEG_A 					0x00
#define SEG_D 					0x08
#define SEG_C 					0x10
#define SEG_E 					0x18

// this should be read from base + 2 if the S508 board is present
#define S508_ID					0x11

// SDLA configuration byte
#define S508_SDLA_CONFIG		0x20

//
// Macros
//

// Various macros for windows
#define	SAYSTATE(x)		SetWindowText(exehwn, x)
#define	SAYNOSTATE()	SetWindowText(exehwn, szProgramHeader)
#define	SAYMSG(x)		AddStringListBox(x)

#define	ADJUST(x)		((x*ca)/CAF)

//
// Global variables
//
HINSTANCE ghinstance;
HWND exehwn, lbhwn, cbhwn, quhwn, hdhwn;

char szProgramHeader[60];
char DebugString[200];

char *our_ini_file			= "sdla.ini";

#ifdef MAKEFORNT
	char *ini502CodeEntry		= "code_514";
#else
	char *ini502CodeEntry		= "code_502";
#endif

char *ini502TestEntry		= "test_code_514";
char *iniCardSectionBase	= "sdla";
char *iniWSEntry			= "window_segment";
char *iniWOEntry			= "window_offset";
char *iniSizeEntry			= "window_size";

char *preparemessages[] = {
"* The card is already in its loading stage!",
"* The card is currently being used!",
"The card is ready to be loaded.",
"* An internal allocation error occurred!"
};

char *ifailedmsg = "The SDLA driver could not be opened! It may not be installed properly, or it couldn't start.";
char *ifailedhdr = "Initialization Failure";

// JP 100 op-code for the NMI
BYTE NMI_code[] = { 0xC3, 0x00, 0x01 };	

// set aside a pretty big buffer for rcode to return text information
char result_text[1000];

BYTE card = 0;			// card to load. Default is 0
WORD window_size;
void *mbuff;
WORD mbuff_size;
char codefname[80], configfname[80];
HWND aphwn = NULL;
BOOL abort_the_mission = FALSE;

BOOL okflag;

WORD final_window_addr = 0xe000;

//
// Function prototypes
//

// entry point
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
// main window
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
// select card "launcher"
void SelCard(void);
// select card dialog
LRESULT CALLBACK SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
// auto-load dialog 
LRESULT CALLBACK AutoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);

// Returns TRUE if "-" appears in hte command line, which meand that user
// wants to issue auto load
BOOL CommandOverride(LPSTR cline);

// Performs the loading process
BOOL DoLoading(void);

// Returns TRUE if S508 found on given i/o
BOOL if_508_present(void);

// Stops the board and enables host access to the board memory
void StopCard(void);

// Sets the hosts 8K window the board will present its memory into
BOOL Set8kWindow(DWORD physical_address);

// Actual code loading is done here ...
BOOL DownloadCode(void);

// Sets the correct window for the logical offset, and returns the offset
// within the window
WORD MapSetZ80Window(WORD logical_offset);

// Reads the given file into the memory, which is dynamically allocated inside the function
int LoadFileToMemory(char *fname);

//
void CopyFileMemoryToBoard(WORD loffset);

// Writes given buffer onto the board starting from given offset
void Z80Write(WORD loffset, BYTE *data, WORD length);

// Compares previously loaded onto the board code with the source
BOOL CompareBoardToFileMemory(WORD loffset);

// Prints out the code version of downloaded code
void SayCodeVersion(void);

// Frees memory allocated for the data read from file
void FreeFileMemory(void);

// Starts the code on the board
void StartCard(void);

// Checks with the driver if the card is configured and if it's not in use
void CheckCardStatus(void);

// Adds the message to the listbox
void AddStringListBox(char far *m);

#ifdef MAKEFORNT
DWORD GetRegistryDWordValue(char *keyname);
#endif


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG msg;
	BOOL fRetVal;
	WNDCLASS wndclass;   // Structure used to register Windows class.
	int title_height;
	HMENU hmenu;
	int i, ca, wid;
	DLGPROC ap;
	BOOL went_ok;
	HDC hdc;
	TEXTMETRIC tm;

	hdc = GetDC(NULL);
	GetTextMetrics(hdc, &tm);
	ReleaseDC(NULL, hdc);
	ca = tm.tmAveCharWidth;
	title_height = GetSystemMetrics(SM_CYCAPTION);
        if (tm.tmHeight > title_height) title_height = tm.tmHeight;

	ghinstance = hInstance;

	if (hPrevInstance != NULL) return(0);

	if (CommandOverride(lpszCmdLine)) {
		if (!APISETUP()) {
			MessageBox(HWND_DESKTOP, ifailedmsg, ifailedhdr, MB_OK | MB_ICONSTOP);
			return(0);
		}
		ap = (DLGPROC)MakeProcInstance((FARPROC)AutoDlg, hInstance);
		aphwn = CreateDialog(hInstance, "AUTOLOAD", HWND_DESKTOP, ap);
		i=0; went_ok = TRUE;
		// right now, there is no other thread to allow aborting
		// during this process, but keep the abort_the_mission
		// variable in here in case it can be set from somewhere
		while (lpszCmdLine[i] != 0 && !abort_the_mission) {
			if (lpszCmdLine[i] == '-') {
				i++;
				if (lpszCmdLine[i] == 0) i--;
				else {
					card = (BYTE)atoi((LPSTR)(lpszCmdLine+i));
					if (card < MAX_CARDS) {
						SetDlgItemInt(aphwn, 110, card, FALSE);
						went_ok &= DoLoading();
					}
				}
			}
			i++;
		}
		result_text[0] = 0;
		if (!went_ok)
			strcat(result_text, "ERROR(S) OCCURRED WHILE LOADING\n");
		if (result_text[0] != 0) {
			strcat(result_text, "It is advised that you run this program again interactively to trace the problem, and to ensure reliable operation of the SDLA card(s).");
			MessageBox(HWND_DESKTOP, result_text, "Loader Message", MB_OK | MB_ICONEXCLAMATION);
		}
		DestroyWindow(aphwn);
		APICLEANUP();
	    return(0);
	}	// if (CommandOverride(lpszCmdLine)) {

	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(ghinstance, "ICON");
	wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = (HBRUSH)GetStockObject( GRAY_BRUSH );
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (!RegisterClass(&wndclass)) {
		return(0);
	}

	exehwn = CreateWindow( szClassName,
				szTitleName,
				WS_OVERLAPPED | WS_MINIMIZE | WS_MINIMIZEBOX | WS_CAPTION | WS_SYSMENU | WS_BORDER,
				CW_USEDEFAULT,
				200,
				ADJUST(400),
				title_height*9,
				HWND_DESKTOP,
				NULL,
				ghinstance,
				NULL);

	if (!exehwn)  return(0);

	if (!APISETUP()) {
		MessageBox(HWND_DESKTOP, ifailedmsg, ifailedhdr, MB_OK | MB_ICONSTOP);
		DestroyWindow(exehwn);
        return(0);
	}

	wid = ADJUST(100);
	cbhwn   = CreateWindow("BUTTON", "Card Select", WS_CHILD | WS_VISIBLE, 2, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	quhwn   = CreateWindow("BUTTON", "Quit", WS_CHILD | WS_VISIBLE, ADJUST(400)-wid-6, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	hdhwn   = CreateWindow("BUTTON", "Load", WS_CHILD | WS_VISIBLE, wid+6, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	lbhwn	= CreateWindow("LISTBOX", "", WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_DISABLENOSCROLL | LBS_NOINTEGRALHEIGHT, 1, title_height+5, ADJUST(396), title_height*6.5, exehwn, NULL, ghinstance, NULL);

	ShowWindow(exehwn, cmdShow);
	UpdateWindow(exehwn);

	hmenu = GetSystemMenu(exehwn, FALSE);
	AppendMenu(hmenu, MF_SEPARATOR, 0, (LPSTR) NULL);
	AppendMenu(hmenu, MF_STRING, IDM_ABOUT, "About...");

	sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
	SAYNOSTATE();

	CheckCardStatus();

	fRetVal = TRUE;
	while (fRetVal)  {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {

			if (msg.message == WM_QUIT)
				fRetVal = FALSE;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return fRetVal;
}

// returns TRUE if dash "-" appears in the command line string
BOOL CommandOverride(LPSTR cline)
{
	int i;
	i=0;
	while (cline[i] != 0) {
		if (cline[i] == '-') return(TRUE);
		i++;
	}
	return(FALSE);
}

//
// Do the actual loading. This function is called from the main window when
// "Load" button is pressed
//
BOOL DoLoading(void)
{
	int rcode;
	char s[80];
	WORD sg, of;
	DWORD ad;

	SAYMSG(".");
	rcode = APIPREPAREFORLOADING(card);
	if (rcode != 2) {
		SAYMSG("* Card currently in use or registration failed!");
		return(FALSE);
	}

	if (!if_508_present) {
		SAYMSG("* Sangoma S508 card not found at specified I/O port!");
		APILOADCOMPLETE(card);
		return(FALSE);
	}

	StopCard();

	sprintf(s, "%s%u", iniCardSectionBase, card);
#ifdef MAKEFORNT
	sg = (WORD)GetRegistryDWordValue(iniWSEntry);
	of = (WORD)GetRegistryDWordValue(iniWOEntry);
#else
	sg = (WORD)GetPrivateProfileInt(s, iniWSEntry, 0xd000, our_ini_file);
	of = (WORD)GetPrivateProfileInt(s, iniWOEntry, 0xd000, our_ini_file);
#endif
	ad = ((DWORD)sg << 4) + (DWORD)of;

	if (!Set8kWindow(ad)) {
		SAYMSG("* Invalid segment for shared memory area!");
		APILOADCOMPLETE(card);
		return(FALSE);
	}
#ifdef MAKEFORNT
	window_size = (WORD)GetRegistryDWordValue(iniSizeEntry);
#else
	window_size = (WORD)GetPrivateProfileInt(s, iniSizeEntry, 0x2000, our_ini_file);
#endif
	GetPrivateProfileString(iniLoaderSection, ini502CodeEntry, "", codefname, 80, our_ini_file);
#if	!defined(PROTO_STREAM) && !defined(PROTO_SDLC)
	GetPrivateProfileString(s, iniConfigEntry, "", configfname, 80, our_ini_file);
#endif

	//sprintf( s, "Segment %04X Offset %04X Size %04X", sg, of, window_size); SAYMSG(s);
	//sprintf(s, "Test code file %s", testcodefname ); SAYMSG(s);
	//sprintf(s, "Code file %s", codefname ); SAYMSG(s);

	if (!DownloadCode()) {
		APILOADCOMPLETE(card);
		return(FALSE);
	}

	APILOADCOMPLETE(card);
	SAYMSG("Finished loading the card.");
	return(TRUE);
}

//
// Returns TRUE if S508 is present
//
BOOL if_508_present(void)
{
	return (S508_ID == APIREADPORTBYTE(card, 2));
}

//
// Stop the 508 board and enable host access to the boards memory
//
void StopCard(void)
{
	APIWRITEPORTBYTE(card, 0, HALT_CPU);
	APIWRITEPORTBYTE(card, 0, MEM_ENABLE);
}

//
// Sets the hosts 8K window the board will present its memory into
//
BOOL Set8kWindow(DWORD physical_address)
{
	BYTE seg, segcode, offcode;

	seg = (BYTE)(physical_address >> 16);
	switch(seg) {
	case 0x0a: segcode = SEG_A; break;
	case 0x0c: segcode = SEG_C; break;
	case 0x0d: segcode = SEG_D; break;
	case 0x0e: segcode = SEG_E; break;
	default:
		return(FALSE);
	}
	offcode = (BYTE)(physical_address >> 13) & 0x0f;
	// set the configuration byte to be written out to I/O port + 1 - this byte consists of the definitions for the
	// PC window and the card memory segment.
	APIWRITEPORTBYTE(card, OFF_PC_ADDR_WS_SELECT, (segcode | offcode));
	return(TRUE);
}

//
// Actual code loading is done here ...
//
BOOL DownloadCode(void)
{
	WORD i;

	// fill the entire 64k memory with 0x00 one window at a time
	for (i=0; i <= 0x0e; i+=2) {
		MapSetZ80Window(i << 12);
		APIZEROWINDOW(card);
	}

	SAYMSG("Downloading code...");
	i = LoadFileToMemory(codefname);
	SAYMSG(codefname);
	if (i) {
		if (i == 1) SAYMSG("* Code file not found!");
		else  SAYMSG("* No memory available to load code!");
		return(FALSE);
	}

	CopyFileMemoryToBoard(CODE_BASE_ADDR);
	SAYMSG("Verifying code...");
	if (!CompareBoardToFileMemory(CODE_BASE_ADDR)) {
		SAYMSG("* Code on board is not identical to original!");
		SAYMSG("* Try another memory address.");
		FreeFileMemory();
		return(FALSE);
	}
	FreeFileMemory();

	// move the NMI code (JP 0x100) onto the card
	Z80Write(0x0000, NMI_code, 3);
	Z80Write(0x0066, NMI_code, 3);

	SayCodeVersion();

	if (configfname[0] == '\0') {
		SAYMSG("Warning: no configuration file specified.");
		APIWRITEBYTE(card, MapSetZ80Window(0x0010), S508_SDLA_CONFIG);
	} else {
		SAYMSG("Downloading configuration file...");
		i = LoadFileToMemory(configfname);
		SAYMSG(configfname);
		if (i)
		{	if (i == 1)
				SAYMSG("* Configuration file not found!");
			else
				SAYMSG("* No memory available to load configuration!");
			return(FALSE);
		}
		// have to set the configuration byte first
		*(unsigned char *)mbuff = S508_SDLA_CONFIG;

		Z80Write(0x0010, mbuff, 1);	// for SDLA config byte

	}
	StartCard();
	MapSetZ80Window(final_window_addr); // set to the mailbox ...

	// 
	// For 508/514 we can't actually copy the cofiguration onto the board
	// instead we do SET_CONFIGURATION command. That in case we have 
	// configuration files
	//
	if (configfname[0] != '\0') 
	{	CBLOCK cb;
		HANDLE hHandle;

		// Wait for code initialization for 3 seconds
		hHandle = CreateEvent( NULL, FALSE, FALSE, "crap"); 
		WaitForSingleObject( hHandle,  3000);

		cb.command = 0x51;
		cb.buffer_length = 0x37;
        CopyMemory( &cb.data[0], ((char*)mbuff + 1 ), 0x37 );
		FreeFileMemory();		

		if(cb.data[0x0a] == 0x01)
		{	// adjust HDLC config byte - we need V.35 interface
			cb.data[0x0a] = 0x80;
		}

		XAPIControl(&cb, card);

		if (cb.return_code != 0x00)
		{	SAYMSG("* Unable to set configuration !!!");
			sprintf(DebugString, "cb.return_code = 0x%x", cb.return_code);
			SAYMSG(DebugString);

			return FALSE;
		}
	}

	return(TRUE);
}

//
// Returns z80 window offset from pc window to access the
// specified logical offset (in range of 0-FFFF), and sets the
// z80 window on the board for a proceeding read/write
// uses the window size specified in sdla.ini for the card to
// determine the appropriate mapping (not hard-coded for 8k)
//
// CALL THIS FUNCTION FOR EACH BYTE TO BE READ/WRITTEN FROM/TO SHARED MEMORY
//
WORD MapSetZ80Window(WORD logical_offset)
{
	WORD physoff;
	BYTE winnum;
	static BYTE last_winnum = 0xff;

	physoff = logical_offset % window_size;
	winnum = (BYTE)(logical_offset / window_size);
	if (last_winnum != winnum) {
		// only set the window if changing it from last time
		APIWRITEPORTBYTE(card, OFF_Z80_WIN_SELECT, winnum);
		last_winnum = winnum;
	}
	return(physoff);
}

// 
// Reads the given file into the memory. The memory is dynamically
// allocated. Must be released later
// 
// Return codes : 0=ok, 1=can't open file, 2=can't allocate memory
//
int LoadFileToMemory(char *fname)
{
	size_t length;
	int handle;

	handle = open(fname, O_RDONLY | O_BINARY);
	if (handle == -1) return(1);

	length = (size_t)filelength(handle);
	mbuff = malloc(length);
	if (mbuff == NULL) { close(handle); return(2); }
	read(handle, mbuff, length);
	close(handle);
	mbuff_size = (WORD)length;
	return(0);
}

// 
void CopyFileMemoryToBoard(WORD loffset)
{
	Z80Write(loffset, mbuff, mbuff_size);
}

//
// Writes given buffer onto the board starting from given offset
//
void Z80Write(WORD loffset, BYTE *data, WORD length)
{
	WORD i;
	for (i=0; i < length; i++)
		APIWRITEBYTE(card, MapSetZ80Window(loffset+i), data[i]);
}

//
// Compares previously loaded onto the board code with the source
//
BOOL CompareBoardToFileMemory(WORD loffset)
{
	WORD i;
	BYTE *src;
	BYTE tmp_byte;
	src = (BYTE *)mbuff;
	for (i=0; i < mbuff_size; i++) {
		tmp_byte = APIREADBYTE(card, MapSetZ80Window(loffset+i));
		if (src[i] != tmp_byte) {
			//
			//
			return(FALSE);
		}
	}
	return(TRUE);
}

// Frees memory allocated for the data read from file
void FreeFileMemory(void)
{
	free(mbuff);
}

//
// Prints out the code version of downloaded code
//
void SayCodeVersion(void)
{
	BYTE code_version[SIZEOF_VER_NO+1];
	WORD i;
	char s[80];

	for (i=0; i < SIZEOF_VER_NO; i++)
		code_version[i] = APIREADBYTE(card, MapSetZ80Window(OFF_CODE_VER_HDLC_CODE+i));
	code_version[i] = '\0';
	sprintf(s, "Code version: %s", code_version);
	SAYMSG(s);
}

//
// Starts the code on the board
//
void StartCard(void)
{
	APIWRITEPORTBYTE(card, 0, START_CPU | MEM_ENABLE);
}


// Adds message to the list box
void AddStringListBox(char far *m)
{
	static UINT count = 0;

	if (aphwn != NULL)  {
		SetDlgItemText(aphwn, 101, m);
		return;
	}

	count++;
	if (count == COUNT_MAX)  {
		SendMessage(lbhwn, LB_DELETESTRING, 0, 0L);
		count--;
	}
	SendMessage(lbhwn, LB_ADDSTRING, 0, (LPARAM)m);
	SendMessage(lbhwn, LB_SETCURSEL, count-1, 0L);
}


LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	HWND h;
	char s[80];

	switch(iMessage) {
	case WM_CREATE:
		return(0);
	case WM_DESTROY:
		APICLEANUP();
		PostQuitMessage(0);
		break;
	case WM_COMMAND:
		h = GET_WM_COMMAND_HWND(wParam, lParam);
		if (h == cbhwn) {
			SelCard();
		} else if (h == quhwn) {
			PostMessage(hWnd, WM_CLOSE, 0, 0L);
		} else if (h == hdhwn) {
			sprintf(s, "LOADING Card %u...", card);
			SAYSTATE(s);
			DoLoading();
			SAYNOSTATE();
		}
		return(0);
	case WM_SYSCOMMAND:
		if (GET_WM_COMMAND_ID(wParam, lParam) == IDM_ABOUT)
			MessageBox(exehwn, aboutstatement, abouttitle, MB_OK | MB_ICONINFORMATION);
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	default:
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	}
	return(1);
}

void SelCard(void)
{
	DLGPROC dproc;
	int i;

	dproc = (DLGPROC)MakeProcInstance((FARPROC)SelCardDlg, ghinstance);
	i=DialogBox(ghinstance, "SELCARD", exehwn, dproc);
	FreeProcInstance((FARPROC)dproc);
	if (i) {
		card = i-1;
		sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
		SAYNOSTATE();
		CheckCardStatus();
	}
}

void CheckCardStatus(void)
{
	char s[80];
	LRESULT rg;

	rg = APIGETCARDS();
	if (rg & (0x01L << card))
		sprintf(s, "* Card %u is currently in use!", card);
	else if (rg & (0x00010000L << card))
		sprintf(s, "Card %u is available for loading.", card);
	else
		sprintf(s, "* No configuration entry exists for card %u!", card);
	SAYMSG(s);
}

LRESULT CALLBACK SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	int n, m, default_card;
	LRESULT c;
	char s[5];
	static int ncard;

	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		c = APIGETCARDS(); m = 0;
		ncard = (int)card;
		for (n=0; n < 16; n++) {
			if (c & (1L << (16+n))) {
				sprintf(s, "%2d", n);
				SendDlgItemMessage(hDlg, 101, CB_ADDSTRING, 0, (LPARAM)s);
				if (ncard == n)
					SendDlgItemMessage(hDlg, 101, CB_SETCURSEL, m, 0);
				m++;
			}
		}
//		default_card = GetPrivateProfileInt(iniHardwareSection, iniCardEntry, 0, our_ini_file);
		default_card = 0;
		sprintf(s, "%u", (UINT)default_card);
		SetDlgItemText(hDlg, 103, s);
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 101:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELENDOK) {
				SendDlgItemMessage(hDlg, 101, CB_GETLBTEXT, (WPARAM)SendDlgItemMessage(hDlg, 101, CB_GETCURSEL, 0, 0), (LPARAM)s);
				ncard = atoi(s);
			}
			break;
		case IDOK:
			if (IsDlgButtonChecked(hDlg, 102)) {
				sprintf(s, "%u", (UINT)ncard);
				//WritePrivateProfileString(iniHardwareSection, iniCardEntry, s, our_ini_file);
			}
			EndDialog(hDlg, ncard+1);
			break;
		case IDCANCEL:
			EndDialog(hDlg, 0);
			break;
		}
	}
	return(0);
}

LRESULT CALLBACK AutoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch(iMessage) {
	case WM_INITDIALOG:
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDCANCEL:
			abort_the_mission = TRUE;
			break;
		}
	}
	return(0);
}


#ifdef MAKEFORNT
DWORD GetRegistryDWordValue(char *keyname)
{
	char s[100];
	union { DWORD d; BYTE db[4]; } data;
	DWORD size;
	LONG e;
	HKEY hkey;

	size = 4;
	data.d = 0;
	sprintf(s, "SYSTEM\\CurrentControlSet\\Services\\SDLA\\Parameters\\sdla%u", card);
	//sprintf(s, "SDLA\\Parameters\\sdla%u\\%s", card, keyname);
	RegOpenKeyEx(HKEY_LOCAL_MACHINE, s, 0, KEY_QUERY_VALUE, &hkey);
	if ((e=RegQueryValueEx(hkey, keyname, NULL, NULL, &(data.db[0]), &size))
		!= ERROR_SUCCESS) {
			sprintf(s, "Registry Error: %ld", e);
			SAYMSG(s);
		}
	RegCloseKey(hkey);
	return(data.d);
}
#endif

