//
// Winxldr514.C - X.25 loader for Sangoma S514 board
//
//
// Changed from Winxldr8.C by David on March 2000
//
#define	STRICT

#define DBG_FLAG 1

//
// Include files
//
#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include "winxldr8.h"


#pragma pack(1)
typedef struct {
	BYTE SDLA_configuration;
	BYTE line_speed_number;
	BYTE T1;
	BYTE T2;
	BYTE N2;
	WORD max_I_field_length;
	BYTE k;
	BYTE T4;
	BYTE auto_modem_err_fixup;
	BYTE auto_HDLC;
	BYTE HDLC_config_options;
	BYTE DTE;
	BYTE packet_window;
	WORD default_packet_size;
	WORD maximum_packet_size;
	WORD lowest_PVC;
	WORD highest_PVC;
	WORD lowest_incomming_channel;
	WORD highest_incomming_channel;
	WORD lowest_two_way_channel;
	WORD highest_two_way_channel;
	WORD lowest_outgoing_channel;
	WORD highest_outgoing_channel;
	WORD X25_config_options;
	BYTE X25_response_options;
	WORD genl_facilities_supported_1;
	WORD genl_facilities_supported_2;
	WORD CCITT_facilities_supported;
	WORD non_X25_facilities_supported;
	WORD CCITT_compatibility;
	BYTE T10_T20;
	BYTE T11_T21;
	BYTE T12_T22;
	BYTE T13_T23;
	BYTE T16_T26;
	BYTE T28;
	BYTE R10_R20;
	BYTE R12_R22;
	BYTE R13_R23;
	UCHAR electrical_interface;
} CONFIG_STRUCT;

//
// #define's
//
#define	MSG_POLL		0x4000
#define	IDM_ABOUT		0x1000
#define	COUNT_MAX		100
#define	MAX_CARDS		16
#define	CAF				7

// Z80 commands 
#define HALT_CPU 		0x00 // halt the CPU
#define START_CPU 		0x02 // start the CPU
#define MEM_ENABLE		0x04 // enable host access to boards memory
#define NMI 			0x10 // issue an NMI to the CPU

// SDLA adapter port constants
#define OFF_PC_ADDR_WS_SELECT 	0x01	// the offset from the card I/O port base address of the adapter
										// segment and PC window select latch
#define OFF_Z80_WIN_SELECT 		0x02	// the offset from the card I/O port base address of the Z80 window
										// select latch
#define SET_8K_WINDOW 			0x01	// a bit to enable 8K window operation
#define RESET_8K_WINDOW 		0xFE	// bit 0 reset to disable 8K window operation

// Card 64K segment address settings 
// bits 3-4 of configuration byte written to port (base address + 1)
#define SEG_A 					0x00
#define SEG_D 					0x08
#define SEG_C 					0x10
#define SEG_E 					0x18

// SDLA configuration byte
#define S508_SDLA_CONFIG		0x20

//
// Macros
//

// Various macros for windows
#define	SAYSTATE(x)		SetWindowText(exehwn, x)
#define	SAYNOSTATE()	SetWindowText(exehwn, szProgramHeader)
#define	SAYMSG(x)		AddStringListBox(x)

#define	ADJUST(x)		((x*ca)/CAF)

//
// Global variables
//
HINSTANCE ghinstance;
HWND exehwn, lbhwn, cbhwn, quhwn, hdhwn;

char szProgramHeader[60];
char DebugString[200];

char *our_ini_file			= "sdla.ini";
//char *ini502CodeEntry		= "code_502";
char *ini502CodeEntry		= "code_514";
char *ini502TestEntry		= "test_code_502";
char *iniCardSectionBase	= "sdla";
char *iniWSEntry			= "window_segment";
char *iniWOEntry			= "window_offset";
char *iniSizeEntry			= "window_size";

char *preparemessages[] = {
"* The card is already in its loading stage!",
"* The card is currently being used!",
"The card is ready to be loaded.",
"* An internal allocation error occurred!"
};

char *ifailedmsg = "The SDLA driver could not be opened! It may not be installed properly, or it couldn't start.";
char *ifailedhdr = "Initialization Failure";

// JP 100 op-code for the NMI
BYTE NMI_code[] = { 0xC3, 0x00, 0x01 };	

// set aside a pretty big buffer for rcode to return text information
char result_text[1000];

BYTE card = 0;			// card to load. Default is 0
WORD window_size;
void *mbuff;
WORD mbuff_size;
char codefname[80], configfname[80];
HWND aphwn = NULL;
BOOL abort_the_mission = FALSE;

BOOL okflag;

WORD final_window_addr = 0xe000;

//
// Function prototypes
//

// entry point
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
// main window
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
// select card "launcher"
void SelCard(void);
// select card dialog
LRESULT CALLBACK SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
// auto-load dialog 
LRESULT CALLBACK AutoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);

// Returns TRUE if "-" appears in hte command line, which meand that user
// wants to issue auto load
BOOL CommandOverride(LPSTR cline);

// Performs the loading process
BOOL DoLoading(void);

// Actual code loading is done here ...
BOOL DownloadCode(void);

// Reads the given file into the memory, which is dynamically allocated inside the function
int LoadFileToMemory(char *fname);

//
void CopyFileMemoryToBoard(WORD loffset);

// Writes given buffer onto the board starting from given offset
void Z80Write(WORD loffset, BYTE *data, WORD length);

// Compares previously loaded onto the board code with the source
BOOL CompareBoardToFileMemory(WORD loffset);

// Prints out the code version of downloaded code
void SayCodeVersion(void);

// Frees memory allocated for the data read from file
void FreeFileMemory(void);

// Starts the code on the board
void StartCard(void);

// Checks with the driver if the card is configured and if it's not in use
void CheckCardStatus(void);

// Adds the message to the listbox
void AddStringListBox(char far *m);

#ifdef MAKEFORNT
DWORD GetRegistryDWordValue(char *keyname);
#endif

static void Debug(UCHAR dbg_flag, PUCHAR pszFormat, ...);

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG msg;
	BOOL fRetVal;
	WNDCLASS wndclass;   // Structure used to register Windows class.
	int title_height;
	HMENU hmenu;
	int i, ca, wid;
	DLGPROC ap;
	BOOL went_ok;
	HDC hdc;
	TEXTMETRIC tm;

	hdc = GetDC(NULL);
	GetTextMetrics(hdc, &tm);
	ReleaseDC(NULL, hdc);
	ca = tm.tmAveCharWidth;
	title_height = GetSystemMetrics(SM_CYCAPTION);
        if (tm.tmHeight > title_height) title_height = tm.tmHeight;

	ghinstance = hInstance;

	if (hPrevInstance != NULL) return(0);

	if (CommandOverride(lpszCmdLine)) {
		if (!APISETUP()) {
			MessageBox(HWND_DESKTOP, ifailedmsg, ifailedhdr, MB_OK | MB_ICONSTOP);
			return(0);
		}
		ap = (DLGPROC)MakeProcInstance((FARPROC)AutoDlg, hInstance);
		aphwn = CreateDialog(hInstance, "AUTOLOAD", HWND_DESKTOP, ap);
		i=0; went_ok = TRUE;
		// right now, there is no other thread to allow aborting
		// during this process, but keep the abort_the_mission
		// variable in here in case it can be set from somewhere
		while (lpszCmdLine[i] != 0 && !abort_the_mission) {
			if (lpszCmdLine[i] == '-') {
				i++;
				if (lpszCmdLine[i] == 0) i--;
				else {
					card = (BYTE)atoi((LPSTR)(lpszCmdLine+i));
					if (card < MAX_CARDS) {
						SetDlgItemInt(aphwn, 110, card, FALSE);
						went_ok &= DoLoading();
					}
				}
			}
			i++;
		}
		result_text[0] = 0;
		if (!went_ok)
			strcat(result_text, "ERROR(S) OCCURRED WHILE LOADING\n");
		if (result_text[0] != 0) {
			strcat(result_text, "It is advised that you run this program again interactively to trace the problem, and to ensure reliable operation of the SDLA card(s).");
			MessageBox(HWND_DESKTOP, result_text, "Loader Message", MB_OK | MB_ICONEXCLAMATION);
		}
		DestroyWindow(aphwn);
		APICLEANUP();
	    return(0);
	}	// if (CommandOverride(lpszCmdLine)) {

	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(ghinstance, "ICON");
	wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = (HBRUSH)GetStockObject( GRAY_BRUSH );
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (!RegisterClass(&wndclass)) {
		return(0);
	}

	exehwn = CreateWindow( szClassName,
				szTitleName,
				WS_OVERLAPPED | WS_MINIMIZE | WS_MINIMIZEBOX | WS_CAPTION | WS_SYSMENU | WS_BORDER,
				CW_USEDEFAULT,
				200,
				ADJUST(400),
				title_height*9,
				HWND_DESKTOP,
				NULL,
				ghinstance,
				NULL);

	if (!exehwn)  return(0);

	if (!APISETUP()) {
		MessageBox(HWND_DESKTOP, ifailedmsg, ifailedhdr, MB_OK | MB_ICONSTOP);
		DestroyWindow(exehwn);
        return(0);
	}

	wid = ADJUST(100);
	cbhwn   = CreateWindow("BUTTON", "Card Select", WS_CHILD | WS_VISIBLE, 2, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	quhwn   = CreateWindow("BUTTON", "Quit", WS_CHILD | WS_VISIBLE, ADJUST(400)-wid-6, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	hdhwn   = CreateWindow("BUTTON", "Load", WS_CHILD | WS_VISIBLE, wid+6, 2, wid, title_height, exehwn, NULL, ghinstance, NULL);
	lbhwn	= CreateWindow("LISTBOX", "", WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_DISABLENOSCROLL | LBS_NOINTEGRALHEIGHT, 1, title_height+5, ADJUST(396), title_height*6.5, exehwn, NULL, ghinstance, NULL);

	ShowWindow(exehwn, cmdShow);
	UpdateWindow(exehwn);

	hmenu = GetSystemMenu(exehwn, FALSE);
	AppendMenu(hmenu, MF_SEPARATOR, 0, (LPSTR) NULL);
	AppendMenu(hmenu, MF_STRING, IDM_ABOUT, "About...");

	sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
	SAYNOSTATE();

	CheckCardStatus();

	fRetVal = TRUE;
	while (fRetVal)  {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {

			if (msg.message == WM_QUIT)
				fRetVal = FALSE;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return fRetVal;
}

// returns TRUE if dash "-" appears in the command line string
BOOL CommandOverride(LPSTR cline)
{
	int i;
	i=0;
	while (cline[i] != 0) {
		if (cline[i] == '-') return(TRUE);
		i++;
	}
	return(FALSE);
}

//
// Do the actual loading. This function is called from the main window when
// "Load" button is pressed
//
BOOL DoLoading(void)
{
	int rcode;
	char s[80];
	USHORT hw_type=0;

	sprintf(s, "%s%u", iniCardSectionBase, card);
	hw_type = (USHORT)GetRegistryDWordValue("HardWareType");
	if(hw_type == 514){
		Debug(DBG_FLAG, "Loader:hw_type == 514\n");
	}else{
		//It's S514 loader refuse to load anything else
		Debug(DBG_FLAG, "Loader:hw_type == 508!!!\n");
		return(FALSE);
	}

	SAYMSG(".");
	rcode = APIPREPAREFORLOADING(card);

	if (rcode != 2) {
		SAYMSG("* Card currently in use or registration failed!");
		return(FALSE);
	}

	GetPrivateProfileString(iniLoaderSection, ini502CodeEntry, "", codefname, 80, our_ini_file);
	Debug(DBG_FLAG, "Loader:codefname :%s\n", codefname);

#if	!defined(PROTO_STREAM) && !defined(PROTO_SDLC)
	GetPrivateProfileString(s, iniConfigEntry, "", configfname, 80, our_ini_file);
#endif

	//sprintf( s, "Segment %04X Offset %04X Size %04X", sg, of, window_size); SAYMSG(s);
	//sprintf(s, "Test code file %s", testcodefname ); SAYMSG(s);
	//sprintf(s, "Code file %s", codefname ); SAYMSG(s);
	if (!DownloadCode()){
		APILOADCOMPLETE(card);
		return(FALSE);
	}

	APILOADCOMPLETE(card);
	SAYMSG("Finished loading the card.");

	return(TRUE);
}


//
// Actual code loading is done here ...
//
BOOL DownloadCode(void)
{
	WORD i;
	UCHAR test=1;
	ULONG HDLC_T1 = 0x66b5;
	HANDLE hDelayHandle;

	//stop the card in case it was running
	XAPIStopS514(card);
	//zero card's memory
	APIZEROWINDOW(card);

	SAYMSG("Downloading code...");
	i = LoadFileToMemory(codefname);
	SAYMSG(codefname);

	Debug(DBG_FLAG, "DownloadCode:LoadFileToMemory :i==%d\n", i);

	if (i)
	{
		if (i == 1) SAYMSG("* Code file not found!");
		else  SAYMSG("* No memory available to load code!");
		return(FALSE);
	}

	Debug(DBG_FLAG, "calling CopyFileMemoryToBoard\n");

	CopyFileMemoryToBoard(CODE_BASE_ADDR);
	SAYMSG("Verifying code...");
	if (!CompareBoardToFileMemory(CODE_BASE_ADDR)) {
		SAYMSG("* Code on board is not identical to original!");
		SAYMSG("* Memory access problem.");
		FreeFileMemory();
		return(FALSE);
	}
	FreeFileMemory();

	// void Z80Write(WORD loffset, BYTE *data, WORD length)
	// move the NMI code (JP 0x100) onto the card
	Z80Write(0x0000, NMI_code, 3);
	Z80Write(0x0066, NMI_code, 3);

	SayCodeVersion();

	if (configfname[0] == '\0') {
		
		SAYMSG("Warning: no configuration file specified.");
		APIWRITEBYTE(card, 0x0010, S508_SDLA_CONFIG);
	}else{

		SAYMSG("Downloading configuration file...");
		i = LoadFileToMemory(configfname);
		SAYMSG(configfname);
		if (i)
		{	if (i == 1)
				SAYMSG("* Configuration file not found!");
			else
				SAYMSG("* No memory available to load configuration!");
			return(FALSE);
		}
	}

	StartCard();

	// Wait for code initialization for 1 second
	hDelayHandle = CreateEvent( NULL, FALSE, FALSE, NULL); 
	WaitForSingleObject( hDelayHandle,  1000);

	// 
	// For 508/514 we can't actually copy the cofiguration onto the board
	// instead we do SET_HDLC_X25_CONFIGURATION (xapi_m.h) command.
	// That in case we have configuration files.
	//
	if (configfname[0] != '\0')
	{
		CBLOCK cb;
		CONFIG_STRUCT * x25_cfg_ptr = (CONFIG_STRUCT *)mbuff;
		
		cb.command = 0x51;//SET_HDLC_X25_CONFIGURATION
		cb.buffer_length = 0x37;//sizeof config structure

		//note that 'electrical_interface' is not copied
        CopyMemory( &cb.data[0], ((char*)mbuff + 1 ), 0x37 );
		FreeFileMemory();		

#if DBG_FLAG
		//print out configuration structure
		for(i=0;i<0x37;i++){
			Debug(DBG_FLAG, "cb.data[0x%x] ==0x%x\n", i, cb.data[i]);
		}
#endif

		//If we need V.35 interface - adjust HDLC config byte.
		Debug(DBG_FLAG, "x25_cfg_ptr->electrical_interface == 0x%x\n",
			x25_cfg_ptr->electrical_interface);

		//x25_cfg_ptr->electrical_interface can be : 0x80 or 0x00
		cb.data[0x0a] |= x25_cfg_ptr->electrical_interface;

		Debug(DBG_FLAG, "cb.data[0x0a] == 0x%x\n", cb.data[0x0a]);

#if DBG_FLAG
		//print out configuration structure after possible update		
		//0x37 == 55 decimal
		for(i=0;i<0x37;i++){
			Debug(DBG_FLAG, "cb.data[0x%x] ==0x%x\n", i, cb.data[i]);
		}
#endif

		XAPIControl(&cb, card);

		Debug(DBG_FLAG, "DownloadCode(): Set HDLC_X25 cfg : cb.return_code == 0x%x\n",
			cb.return_code);

		if(cb.return_code){
			
			sprintf(result_text, "* Unable to set configuration!!! (rc : 0x%X)",
				cb.return_code);
			SAYMSG(result_text);
			return FALSE;
		}
#if DBG_FLAG
		//read back the configuration to see if it's the same as was set.
		else
		{
			for(i=0;i<0x37;i++)
				cb.data[i] = 0;

			cb.command = 0x50;
			cb.buffer_length = 0x00;
			XAPIControl(&cb, card);
			if(cb.return_code == 0x00){

				Debug(DBG_FLAG, "READ_HDLC_X25_CONFIGURATION:\n");
				for(i=0;i<0x37;i++)	{
					Debug(DBG_FLAG, "cb.data[0x%x] ==0x%x\n", i, cb.data[i]);
				}
			}else{
				Debug(DBG_FLAG, "READ_HDLC_X25_CONFIGURATION failed!!\n");
			}
		}
#endif

	}//if (configfname[0] != '\0')

	return(TRUE);
}


// 
// Reads the given file into the memory. The memory is dynamically
// allocated. Must be released later
// 
// Return codes : 0=ok, 1=can't open file, 2=can't allocate memory
//
int LoadFileToMemory(char *fname)
{
	size_t length;
	int handle;

	handle = open(fname, O_RDONLY | O_BINARY);
	if (handle == -1) return(1);

	length = (size_t)filelength(handle);
	mbuff = malloc(length);
	if (mbuff == NULL) { close(handle); return(2); }
	read(handle, mbuff, length);
	close(handle);
	mbuff_size = (WORD)length;
	return(0);
}

// 
void CopyFileMemoryToBoard(WORD loffset)
{
	Z80Write(loffset, mbuff, mbuff_size);
}

//
// Writes given buffer onto the board starting from given offset
//
void Z80Write(WORD loffset, BYTE *data, WORD length)
{
	WORD i;

	for (i=0; i < length; i++)
	{
//		Debug(DBG_FLAG, "Z80Write:i==%d\n", i);
		APIWRITEBYTE(card, (loffset+i), data[i]);
	}
}

//
// Compares previously loaded onto the board code with the source
//
BOOL CompareBoardToFileMemory(WORD loffset)
{
	WORD i;
	BYTE *src;
	BYTE tmp_byte;
	src = (BYTE *)mbuff;
	for (i=0; i < mbuff_size; i++) {
		tmp_byte = APIREADBYTE(card, (loffset+i) );
		if (src[i] != tmp_byte) {
			//
			//
			return(FALSE);
		}
	}
	return(TRUE);
}

// Frees memory allocated for the data read from file
void FreeFileMemory(void)
{
	free(mbuff);
}

//
// Prints out the code version of downloaded code
//
void SayCodeVersion(void)
{
	BYTE code_version[SIZEOF_VER_NO+1];
	WORD i;
	char s[80];

	for (i=0; i < SIZEOF_VER_NO; i++)
		code_version[i] = APIREADBYTE(card, (OFF_CODE_VER_HDLC_CODE+i) );
	code_version[i] = '\0';
	sprintf(s, "Code version: %s", code_version);
	SAYMSG(s);
}

//
// Starts the code on the board
//
void StartCard(void)
{
	XAPIStartS514(card);
}


// Adds message to the list box
void AddStringListBox(char far *m)
{
	static UINT count = 0;

	if (aphwn != NULL)  {
		SetDlgItemText(aphwn, 101, m);
		return;
	}

	count++;
	if (count == COUNT_MAX)  {
		SendMessage(lbhwn, LB_DELETESTRING, 0, 0L);
		count--;
	}
	SendMessage(lbhwn, LB_ADDSTRING, 0, (LPARAM)m);
	SendMessage(lbhwn, LB_SETCURSEL, count-1, 0L);
}


LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	HWND h;
	char s[80];

	switch(iMessage) {
	case WM_CREATE:
		return(0);
	case WM_DESTROY:
		APICLEANUP();
		PostQuitMessage(0);
		break;
	case WM_COMMAND:
		h = GET_WM_COMMAND_HWND(wParam, lParam);
		if (h == cbhwn) {
			SelCard();
		} else if (h == quhwn) {
			PostMessage(hWnd, WM_CLOSE, 0, 0L);
		} else if (h == hdhwn) {
			sprintf(s, "LOADING Card %u...", card);
			SAYSTATE(s);
			DoLoading();
			SAYNOSTATE();
		}
		return(0);
	case WM_SYSCOMMAND:
		if (GET_WM_COMMAND_ID(wParam, lParam) == IDM_ABOUT)
			MessageBox(exehwn, aboutstatement, abouttitle, MB_OK | MB_ICONINFORMATION);
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	default:
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	}
	return(1);
}

void SelCard(void)
{
	DLGPROC dproc;
	int i;

	dproc = (DLGPROC)MakeProcInstance((FARPROC)SelCardDlg, ghinstance);
	i=DialogBox(ghinstance, "SELCARD", exehwn, dproc);
	FreeProcInstance((FARPROC)dproc);
	if (i) {
		card = i-1;
		sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
		SAYNOSTATE();
		CheckCardStatus();
	}
}

void CheckCardStatus(void)
{
	char s[80];
	LRESULT rg;

	rg = APIGETCARDS();
	if (rg & (0x01L << card))
		sprintf(s, "* Card %u is currently in use!", card);
	else if (rg & (0x00010000L << card))
		sprintf(s, "Card %u is available for loading.", card);
	else
		sprintf(s, "* No configuration entry exists for card %u!", card);
	SAYMSG(s);
}

LRESULT CALLBACK SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	int n, m, default_card;
	LRESULT c;
	char s[5];
	static int ncard;

	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		c = APIGETCARDS(); m = 0;
		ncard = (int)card;
		for (n=0; n < 16; n++) {
			if (c & (1L << (16+n))) {
				sprintf(s, "%2d", n);
				SendDlgItemMessage(hDlg, 101, CB_ADDSTRING, 0, (LPARAM)s);
				if (ncard == n)
					SendDlgItemMessage(hDlg, 101, CB_SETCURSEL, m, 0);
				m++;
			}
		}
//		default_card = GetPrivateProfileInt(iniHardwareSection, iniCardEntry, 0, our_ini_file);
		default_card = 0;
		sprintf(s, "%u", (UINT)default_card);
		SetDlgItemText(hDlg, 103, s);
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 101:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELENDOK) {
				SendDlgItemMessage(hDlg, 101, CB_GETLBTEXT, (WPARAM)SendDlgItemMessage(hDlg, 101, CB_GETCURSEL, 0, 0), (LPARAM)s);
				ncard = atoi(s);
			}
			break;
		case IDOK:
			if (IsDlgButtonChecked(hDlg, 102)) {
				sprintf(s, "%u", (UINT)ncard);
				//WritePrivateProfileString(iniHardwareSection, iniCardEntry, s, our_ini_file);
			}
			EndDialog(hDlg, ncard+1);
			break;
		case IDCANCEL:
			EndDialog(hDlg, 0);
			break;
		}
	}
	return(0);
}

LRESULT CALLBACK AutoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch(iMessage) {
	case WM_INITDIALOG:
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDCANCEL:
			abort_the_mission = TRUE;
			break;
		}
	}
	return(0);
}


#ifdef MAKEFORNT
DWORD GetRegistryDWordValue(char *keyname)
{
	char s[100];
	union { DWORD d; BYTE db[4]; } data;
	DWORD size;
	LONG e;
	HKEY hkey;

	size = 4;
	data.d = 0;
	sprintf(s, "SYSTEM\\CurrentControlSet\\Services\\SDLA\\Parameters\\sdla%u", card);
	//sprintf(s, "SDLA\\Parameters\\sdla%u\\%s", card, keyname);
	RegOpenKeyEx(HKEY_LOCAL_MACHINE, s, 0, KEY_QUERY_VALUE, &hkey);
	if ((e=RegQueryValueEx(hkey, keyname, NULL, NULL, &(data.db[0]), &size))
		!= ERROR_SUCCESS) {
			sprintf(s, "Registry Error: %ld", e);
			SAYMSG(s);
		}
	RegCloseKey(hkey);
	return(data.d);
}
#endif


#define MAX_DEBUG_STRING 512

static void Debug(UCHAR dbg_flag, PUCHAR pszFormat, ...)
{
	if(dbg_flag)
	{	UCHAR	str[MAX_DEBUG_STRING];
		va_list	vaArg;

		va_start (vaArg, pszFormat);
		//vsprintf((char *)str, (char *)pszFormat, vaArg);
		_vsnprintf((char *)str, MAX_DEBUG_STRING, (char *)pszFormat, vaArg);
		va_end (vaArg);

		OutputDebugString( (char *)str );
	}
}
