// Control.cpp -- IOCTL handlers for sdla driver

#include "stddcls.h"
#include "driver.h"
//#include "ioctls.h"
#include "sdla.h"
#include "sdlapci.h"


///////////////////////////////////////////////////////////////////////////////

#pragma PAGEDCODE

NTSTATUS DispatchControl(PDEVICE_OBJECT fdo, PIRP Irp)
{							// DispatchControl
	PAGED_CODE();
	//PDEVICE_EXTENSION extension = (PDEVICE_EXTENSION) fdo->DeviceExtension;
	PDEVICE_EXTENSION extension;
//?????????????????????????
	UCHAR x;
	XFER_BYTE_CONTROL *OutputBuffer;
	SDLA_BLOCKOP *block;
	CARD_TRANSACTION *trans;
	UCHAR cardnumber;
	ULONG c, *lr, *tmp;
	ULONG offset;
	USHORT ind;

//?????????????????????????
	NTSTATUS status;

	
	PIO_STACK_LOCATION stack = IoGetCurrentIrpStackLocation(Irp);
	ULONG code = stack->Parameters.DeviceIoControl.IoControlCode;
	
//	DbgPrint("DispatchControl()\n");

	switch (code)
	{
		
	case IOCTL_SDLA_DUMMY:
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(ULONG);
		tmp = (PULONG) Irp->AssociatedIrp.SystemBuffer;
		break;
		
	case IOCTL_SDLA_GET_CARDS:
#ifdef DEBG
		DbgPrint("IOCTL_SDLA_GET_CARDS ...\n");

		for(ind = 0; ind < MAX_CARDS; ind++)
		{
			DbgPrint("sdla[%d] will use device:%d\n", ind, sdla[ind]);
			DbgPrint("dev_array[%d] : 0x%X\n", ind, dev_array[ind]);
		}
#endif
		c = 0;
		for (cardnumber=0; cardnumber < MAX_CARDS; cardnumber++) {
			if (cardinstances[cardnumber] > 0) c |= (1L << cardnumber);
		}
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(ULONG);
		lr = (PULONG) Irp->AssociatedIrp.SystemBuffer;
		*lr = c | ((ULONG)available_cards << 16);
#ifdef DEBG
		DbgPrint("Returning 0x%08X from IOCTL_SDLA_GET_CARDS.\n", *lr);
//		sprintf(globalLogString, "Returning 0x%08X from IOCTL_SDLA_GET_CARDS.\0", *lr);
//		StartUpLog();
#endif
		break;
		
	case IOCTL_SDLA_REGISTER:

#ifdef DEBG
		DbgPrint("IOCTL_SDLA_REGISTER ...\n");
#endif
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(CARD_TRANSACTION);
		trans = (CARD_TRANSACTION *) Irp->AssociatedIrp.SystemBuffer;
		// get card number and make sure its valid
		if ((cardnumber = trans->cardnumber) >= MAX_CARDS)
		{ 
			trans->rcode = SDLAR_NOT_REGISTERED;
			break; 
		}
		if (blockregistration[cardnumber])
		{
			trans->rcode = SDLAR_CARD_BLOCKED;
			break;
		}
		if (cardinstances[cardnumber] == 0)
		{
			cardinstances[cardnumber]++;
			trans->rcode = SDLAR_ALLOCATED;
#ifdef DEBG
   			DbgPrint("Returning %d from IOCTL_SDLA_REGISTER.\n", trans->rcode );
//			sprintf(globalLogString, "Returning %d from IOCTL_SDLA_REGISTER.\0", trans->rcode);
//			StartUpLog();
#endif	
			break;
		}
		cardinstances[cardnumber]++;
		trans->rcode = SDLAR_REGISTERED;

#ifdef DEBG
   		DbgPrint("SDLAR_REGISTERED:Returning %d from IOCTL_SDLA_REGISTER.\n", trans->rcode );
#endif	
		break;
		
	case IOCTL_SDLA_DEREGISTER:

#ifdef DEBG
		DbgPrint("IOCTL_SDLA_DEREGISTER ...\n");
#endif
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(CARD_TRANSACTION);
		trans = (CARD_TRANSACTION *) Irp->AssociatedIrp.SystemBuffer;
		// get card number and make sure its valid
		if ((cardnumber=trans->cardnumber) >= MAX_CARDS) { 
			trans->rcode = SDLAR_NOT_REGISTERED; break; 
		}
		if (cardinstances[cardnumber] > 0)
		{
			cardinstances[cardnumber]--;
			if (cardinstances[cardnumber] == 0)
			{
				trans->rcode = SDLAR_RELEASED;
#ifdef DEBG
    			DbgPrint("Returning %d from IOCTL_SDLA_DEREGISTER.\n", trans->rcode );
#endif	
				break;
			}
			trans->rcode = SDLAR_DEREGISTERED;
			break;
#ifdef DEBG
    		DbgPrint("Returning %d from IOCTL_SDLA_DEREGISTER.\n", trans->rcode );
#endif	
		}

#ifdef DEBG
   		DbgPrint("Returning %d from IOCTL_SDLA_DEREGISTER.\n", trans->rcode );
//		sprintf(globalLogString, "Returning %d from IOCTL_SDLA_DEREGISTER.\0", trans->rcode);
//		StartUpLog();
#endif	
		trans->rcode = SDLAR_NOT_REGISTERED;
		break;
		
	case IOCTL_SDLA_PREPARE_TO_LOAD:

#ifdef DEBG
		DbgPrint("IOCTL_SDLA_PREPARE_TO_LOAD start\n");
#endif	
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(CARD_TRANSACTION);
		trans = (CARD_TRANSACTION *) Irp->AssociatedIrp.SystemBuffer;
		// get card number and make sure its valid
		if ((cardnumber=trans->cardnumber) >= MAX_CARDS) { 
			trans->rcode = SDLAR_NOT_REGISTERED; break; 
		}

		extension = dev_array[cardnumber];

		if (blockregistration[cardnumber])
			trans->rcode = SDLAR_CARD_BLOCKED;
		else
			if (cardinstances[cardnumber] == 0)
			{
				if (!InitializeCard514(extension))
					trans->rcode = SDLAR_ERROR_PREPARING;
				else
				{
					blockregistration[cardnumber] = 1;
					trans->rcode = SDLAR_PREPARED;
				}
			}
			else
				trans->rcode = SDLAR_REGISTERED;
		break;

	case IOCTL_SDLA_LOAD_COMPLETE:
#ifdef DEBG
		DbgPrint("IOCTL_SDLA_LOAD_COMPLETE\n");
#endif	
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(CARD_TRANSACTION);
		trans = (CARD_TRANSACTION *) Irp->AssociatedIrp.SystemBuffer;
		// get card number and make sure its valid
		if ((cardnumber=trans->cardnumber) >= MAX_CARDS)
		{ 	trans->rcode = SDLAR_NOT_REGISTERED;
			break; 
		}
		if (blockregistration[cardnumber])
		{
			blockregistration[cardnumber] = 0;
			trans->rcode = SDLAR_RELEASED;
		}
		else
			trans->rcode = SDLAR_NOT_PREPARED;
		break;
		
	case IOCTL_SDLA_READ_BYTE:
#ifdef DEBG
//		DbgPrint("IOCTL_SDLA_READ_BYTE\n");
#endif	
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
		offset = OutputBuffer->offset;
		extension = dev_array[cardnumber];
		
		OutputBuffer->value = READ_REGISTER_UCHAR((PUCHAR)(extension->BaseMemoryAddress+offset));

		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;
		
	case IOCTL_SDLA_WRITE_BYTE:
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
		offset = OutputBuffer->offset;
		extension = dev_array[cardnumber];
#ifdef DEBG
//		DbgPrint("IOCTL_SDLA_WRITE_BYTE offset = 0x%x\n", offset);
#endif	
		WRITE_REGISTER_UCHAR((PUCHAR)(extension->BaseMemoryAddress+offset), OutputBuffer->value);

		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;
		
	case IOCTL_SDLA_READ_BLOCK:
		block = (SDLA_BLOCKOP *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = block->card_number;
		offset = block->offset;
		extension = dev_array[cardnumber];
#ifdef DEBG
//		DbgPrint("IOCTL_SDLA_READ_BLOCK offset = 0x%x\n", offset);
#endif	
		READ_REGISTER_BUFFER_UCHAR( (PUCHAR)(extension->BaseMemoryAddress+offset),
									(PUCHAR)block->buffer,
									(ULONG)block->num_bytes);
		
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(SDLA_BLOCKOP);
		break;
		
	case IOCTL_SDLA_WRITE_BLOCK:
		block = (SDLA_BLOCKOP *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = block->card_number;
		offset = block->offset;
		extension = dev_array[cardnumber];
#ifdef DEBG
//		DbgPrint("IOCTL_SDLA_WRITE_BLOCK offset = 0x%x\n", offset);
#endif	
		WRITE_REGISTER_BUFFER_UCHAR((PUCHAR)(extension->BaseMemoryAddress+offset),
									(PUCHAR)block->buffer,
									(ULONG)block->num_bytes);
		
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(SDLA_BLOCKOP);
		break;
		
	case IOCTL_SDLA_ZERO_WINDOW:
#ifdef DEBG
		DbgPrint("IOCTL_SDLA_ZERO_WINDOW\n");
#endif	
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(CARD_TRANSACTION);
		trans = (CARD_TRANSACTION *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = trans->cardnumber;
		extension = dev_array[cardnumber];

		for (offset=0; offset < MAX_SIZEOF_S514_MEMORY; offset++)
			WRITE_REGISTER_UCHAR((extension->BaseMemoryAddress+offset), 0UL);
		
		break;
		
	case IOCTL_SDLA_READ_PORT_BYTE:
#ifdef DEBG
		DbgPrint("IOCTL_SDLA_READ_PORT_BYTE\n");
#endif	
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
/*		if (hw_type[cardnumber] == 508)
		{
			offset = OutputBuffer->offset;
			OutputBuffer->value = READ_PORT_UCHAR((PUCHAR)(portmem[cardnumber]+offset));
		}
*/
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		
		break;
		
	case IOCTL_SDLA_WRITE_PORT_BYTE:
#ifdef DEBG
		DbgPrint("IOCTL_SDLA_WRITE_PORT_BYTE\n");
#endif	
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
/*
		if (hw_type[cardnumber] == 508)
		{
			offset = OutputBuffer->offset;
			WRITE_PORT_UCHAR((PUCHAR)(portmem[cardnumber]+offset), OutputBuffer->value);
		}
*/
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;

	case IOCTL_START_S514:
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
		extension = dev_array[cardnumber];
#ifdef DEBG
		DbgPrint("IOCTL_START_S514:cardnumber = %d\n", cardnumber);
#endif	
		WRITE_REGISTER_UCHAR((PUCHAR)(extension->Memory_S514_CTRL_REG_BYTE), S514_CPU_START);

		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;
		
	case IOCTL_STOP_S514:
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		cardnumber = OutputBuffer->cardnumber;
		extension = dev_array[cardnumber];
#ifdef DEBG
		DbgPrint("IOCTL_STOP_S514:cardnumber = %d\n", cardnumber);
#endif	

		WRITE_REGISTER_UCHAR((PUCHAR)(extension->Memory_S514_CTRL_REG_BYTE), S514_CPU_HALT);
		
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;

	case IOCTL_GET_SDLA_DEVICES:
#ifdef DEBG
		DbgPrint("IOCTL_GET_SDLA_DEVICES\n");
#endif	
		//returns number of sdla devices available
		OutputBuffer = (XFER_BYTE_CONTROL *) Irp->AssociatedIrp.SystemBuffer;
		OutputBuffer->value = (UCHAR)detected_devices_count;
#ifdef DEBG
		DbgPrint("detected_devices_count = %d\n", OutputBuffer->value);
#endif	
		Irp->IoStatus.Status = STATUS_SUCCESS;
		Irp->IoStatus.Information = sizeof(XFER_BYTE_CONTROL);
		break;
		
	default:
		status = STATUS_INVALID_DEVICE_REQUEST;
		break;
	}						// process request

	status = Irp->IoStatus.Status;
//	IoReleaseRemoveLock(&extension->RemoveLock, Irp);
//	return CompleteRequest(Irp, status, info);
    IoCompleteRequest(Irp, 0);
    return(status);

}							// DispatchControl



BOOLEAN InitializeCard514(PDEVICE_EXTENSION extension)
{
	ULONG MemLengthInBytes, RegisterLengthInBytes;
    NTSTATUS code;
	ULONG reg_val = 0x00;

	if(extension->CPU == CPU_A)
	{
		//
		//Enable memory on CPU A. Move to offset PCI_MAP0_DWORD(0x40) from BaseRegisterAddress.
		//
#ifdef DEBG
		DbgPrint("CPU_A\n");
		DbgPrint("s514 Memory   at 0x%x\n", extension->BaseMemoryAddress);
		DbgPrint("s514 Register at 0x%x\n\n", extension->BaseRegisterAddress);
#endif
		
		reg_val = READ_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP0_DWORD));
		
#ifdef DEBG
		DbgPrint("Enable Mem register Before OR: 0x%X\n", reg_val);
#endif
		
		reg_val |= PCI_MEMORY_ENABLE;
		WRITE_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP0_DWORD), reg_val);
		
#ifdef DEBG
		reg_val = READ_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP0_DWORD));
		DbgPrint("Enable Mem register After OR: 0x%X\n", reg_val);
		DbgPrint("PCI_interrupt_configuration : 0x%X\n", 
			*(PULONG)(extension->BaseRegisterAddress + PCI_INT_CONFIG));//0x4C
#endif
	}else
	{
		//
		//Enable memory on CPU B. Move to offset PCI_MAP1_DWORD(0x44) from BaseRegisterAddress.
		//
#ifdef DEBG
		DbgPrint("CPU_B\n");
		DbgPrint("s514 Memory   at 0x%x\n", extension->BaseMemoryAddress);
		DbgPrint("s514 Register at 0x%x\n\n", extension->BaseRegisterAddress);
#endif

		reg_val = READ_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP1_DWORD));

#ifdef DEBG
		DbgPrint("Enable Mem register Before OR: 0x%X\n", reg_val);
#endif

		reg_val |= PCI_MEMORY_ENABLE;
		WRITE_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP1_DWORD), reg_val);

#ifdef DEBG
		reg_val = READ_REGISTER_ULONG((PULONG)(extension->BaseRegisterAddress + PCI_MAP1_DWORD));
		DbgPrint("Enable Mem register After OR: 0x%X\n", reg_val);
		DbgPrint("PCI_interrupt_configuration : 0x%X\n", 
			*(PULONG)(extension->BaseRegisterAddress + PCI_INT_CONFIG));//0x4C
#endif
	}
	
	return (DoMemoryTest(extension));
}


BOOLEAN DoMemoryTest(PDEVICE_EXTENSION extension)
{
	UCHAR indx, test_byte = 0x00;

	sprintf(globalLogString, "Entering Memory Test...\0", test_byte);
	StartUpLog();

#ifdef DEBG
	DbgPrint("Doing Mem Test...\n");
#endif

	for(indx = 0x00; indx < 0xF; indx++)
	{
		WRITE_REGISTER_UCHAR( (PUCHAR)(extension->BaseMemoryAddress+indx), indx);
#ifdef DEBG
		DbgPrint("WRITE_BYTE:indx = 0x%x \n", indx);
		sprintf(globalLogString, "WRITE_BYTE:indx = 0x%x \0", indx);
		StartUpLog();
#endif

		test_byte = READ_REGISTER_UCHAR( (PUCHAR)(extension->BaseMemoryAddress+indx) );

#ifdef DEBG
		DbgPrint("READ_BYTE:test_byte = 0x%x \n", test_byte);
		sprintf(globalLogString, "READ_BYTE:test_byte = 0x%x \0", test_byte);
		StartUpLog();
#endif

		if( indx != test_byte )
		{	DbgPrint("MemTest Failed!!:indx = 0x%x test_byte = 0x%x \n", indx, test_byte);
			sprintf(globalLogString, "Memory Test Failed!!:indx = 0x%x test_byte = 0x%x \0", indx, test_byte);
			StartUpLog();
			return FALSE;
		}
	}

	sprintf(globalLogString, "Passed Memory Test.\0", test_byte);
	StartUpLog();

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////

#pragma LOCKEDCODE

NTSTATUS DispatchInternalControl(PDEVICE_OBJECT fdo, PIRP Irp)
{							// DispatchInternalControl
	PDEVICE_EXTENSION pdx = (PDEVICE_EXTENSION) fdo->DeviceExtension;
	IoSkipCurrentIrpStackLocation(Irp);
	return IoCallDriver(pdx->LowerDeviceObject, Irp);
}							// DispatchInternalControl


void StartUpLog(void)
{
	NTSTATUS NT_status;
	UNICODE_STRING FullFileName;
	WCHAR * wchar_string = L"\\SystemRoot\\system32\\drivers\\sdlalog.wri";
	HANDLE  NtFileHandle;

	UCHAR str[200];
	PUCHAR ptr_log_str = (PUCHAR)&globalLogString[0];
	PUCHAR ptr_str = &str[0];

	OBJECT_ATTRIBUTES ObjectAttributes; 
	IO_STATUS_BLOCK IoStatus;
	TIME_FIELDS TimeFieldes;
	LARGE_INTEGER  CurrentSystemTime, CurrentLocalTime;

	RtlInitUnicodeString(&FullFileName, wchar_string);

    InitializeObjectAttributes ( &ObjectAttributes, 
                                &FullFileName, 
                                OBJ_CASE_INSENSITIVE, 
                                NULL, 
								NULL );

	NT_status = ZwCreateFile(   &NtFileHandle, 
	                            SYNCHRONIZE | FILE_READ_DATA | FILE_APPEND_DATA, 
	                            &ObjectAttributes, 
		                        &IoStatus, 
			                    NULL,                          // alloc size = none 
				                FILE_ATTRIBUTE_NORMAL, 
					            FILE_SHARE_READ | FILE_SHARE_WRITE, 
						        FILE_OPEN_IF, 
							    FILE_SYNCHRONOUS_IO_NONALERT, 
								NULL,  // eabuffer 
								0 );   // ealength 

	if ( !NT_SUCCESS( NT_status ) ) 
	{ 
		#ifdef DEBG
		DbgPrint("ZwCreateFile failed\n");
		#endif	
		return;
	} 
			
	KeQuerySystemTime( &CurrentSystemTime );
 
//	ExSystemTimeToLocalTime(&CurrentSystemTime,	&CurrentLocalTime );

//	RtlTimeToTimeFields( &CurrentLocalTime, &TimeFieldes );
	RtlTimeToTimeFields( &CurrentSystemTime, &TimeFieldes );

//	#ifdef DEBG
//	DbgPrint("TimeFieldes.Day =%u\n", TimeFieldes.Day);
//	DbgPrint("TimeFieldes.Hour =%u\n", TimeFieldes.Hour);
//	DbgPrint("TimeFieldes.Minute =%u\n", TimeFieldes.Minute);
//	DbgPrint("TimeFieldes.Second =%u\n", TimeFieldes.Second);
//	DbgPrint("TimeFieldes.Weekday =%u\n", TimeFieldes.Weekday);
//	#endif
	memset( str, '\0', 190);

	sprintf((PCHAR)str,"\n%u/%u/%u %u:%u:%u ", TimeFieldes.Day, TimeFieldes.Month, TimeFieldes.Year,
					TimeFieldes.Hour, TimeFieldes.Minute, TimeFieldes.Second);
	strcat((PCHAR)str, (PCHAR)ptr_log_str);

	#ifdef DEBG
	DbgPrint((PCHAR)str);
	#endif

	NT_status = ZwWriteFile(  NtFileHandle,
							  NULL,						
							  NULL,						
							  NtFileHandle,				
							  &IoStatus,
							  (PVOID)ptr_str, 
							  strlen((PCHAR)str),
							  NULL,						
							  NULL						
							);

	ZwClose( NtFileHandle );
}
