// Declarations for sdla driver

#ifndef DRIVER_H
#define DRIVER_H

#define DRIVERNAME "SDLA"				// for use in messages
#define LDRIVERNAME L"SDLA"				// for use in UNICODE string constants

#define DEBG

///////////////////////////////////////////////////////////////////////////////
// Device extension structure

enum DEVSTATE {
	STOPPED,								// device stopped
	WORKING,								// started and working
	PENDINGSTOP,							// stop pending
	PENDINGREMOVE,							// remove pending
	SURPRISEREMOVED,						// removed by surprise
	REMOVED,								// removed
	};

typedef struct _DEVICE_EXTENSION {
	PDEVICE_OBJECT DeviceObject;			// device object this extension belongs to
	PDEVICE_OBJECT LowerDeviceObject;		// next lower driver in same stack
	PDEVICE_OBJECT Pdo;						// the PDO
	IO_REMOVE_LOCK RemoveLock;				// removal control locking structure
	UNICODE_STRING ifname;					// interface name
	DEVSTATE state;							// current state of device
	DEVSTATE prevstate;						// state prior to removal query
	DEVICE_POWER_STATE devpower;			// current device power state
	SYSTEM_POWER_STATE syspower;			// current system power state
	DEVICE_CAPABILITIES devcaps;			// copy of most recent device capabilities
	LONG handles;							// # open handles
	PUCHAR membase;							// mapped memory base address
	ULONG memsize;							// size of mapped memory area

	// TODO add additional per-device declarations

	PUCHAR BaseRegisterAddress;	    //to enable memory on 514
	PUCHAR BaseMemoryAddress;		//memory on 514
	PUCHAR Memory_S514_CTRL_REG_BYTE;

#ifdef DEBG
	ULONG Count;
#endif
	UCHAR CPU;

} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

///////////////////////////////////////////////////////////////////////////////
// Global functions

VOID RemoveDevice(IN PDEVICE_OBJECT fdo);
NTSTATUS CompleteRequest(IN PIRP Irp, IN NTSTATUS status, IN ULONG_PTR info);
NTSTATUS CompleteRequest(IN PIRP Irp, IN NTSTATUS status);
NTSTATUS ForwardAndWait(IN PDEVICE_OBJECT fdo, IN PIRP Irp);
NTSTATUS SendDeviceSetPower(PDEVICE_EXTENSION fdo, DEVICE_POWER_STATE state, BOOLEAN wait = FALSE);
VOID SendAsyncNotification(PVOID context);
VOID EnableAllInterfaces(PDEVICE_EXTENSION pdx, BOOLEAN enable);
VOID DeregisterAllInterfaces(PDEVICE_EXTENSION pdx);
NTSTATUS StartDevice(PDEVICE_OBJECT fdo, PCM_PARTIAL_RESOURCE_LIST raw, PCM_PARTIAL_RESOURCE_LIST translated);
VOID StopDevice(PDEVICE_OBJECT fdo, BOOLEAN oktouch = FALSE);

//various functions
BOOLEAN InitializeCard514(PDEVICE_EXTENSION extension);
BOOLEAN DoMemoryTest(PDEVICE_EXTENSION extension);
void StartUpLog(void);

// I/O request handlers

NTSTATUS DispatchCreate(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS DispatchClose(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS DispatchControl(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS DispatchInternalControl(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS DispatchPower(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS DispatchPnp(PDEVICE_OBJECT fdo, PIRP Irp);

#define MAX_CARDS 16

extern BOOLEAN win98;
extern UNICODE_STRING servkey;
extern USHORT devcount, detected_devices_count;

extern PDEVICE_EXTENSION dev_array[MAX_CARDS];
extern PDEVICE_EXTENSION detected_devices_array[MAX_CARDS];

extern USHORT cardinstances[MAX_CARDS];
extern UCHAR blockregistration[MAX_CARDS];
extern SHORT sdla[MAX_CARDS];
extern USHORT available_cards;

// used for storing log strings
extern char globalLogString[300];


#define CPU_A	0
#define CPU_B	1

//this are used for dynamic resource parsing on two platforms supported
//by the driver.
#define S514_1CPU_RES_COUNT_WIN_2000							5
#define S514_2CPU_RES_COUNT_WIN_2000							7
#define BASE_REGISTER_ADDRESS_RESOURCE_NUMBER_WIN_2000			0
#define BASE_MEMORY_ADDRESS_CPU_A_RESOURCE_NUMBER_WIN_2000		2
#define BASE_MEMORY_ADDRESS_CPU_B_RESOURCE_NUMBER_WIN_2000		4//resource 6 is IRQ


#define S514_1CPU_RES_COUNT_WIN_98								3
#define S514_2CPU_RES_COUNT_WIN_98								4
#define BASE_REGISTER_ADDRESS_RESOURCE_NUMBER_WIN_98			1//resource 0 is IRQ
#define BASE_MEMORY_ADDRESS_CPU_A_RESOURCE_NUMBER_WIN_98		2
#define BASE_MEMORY_ADDRESS_CPU_B_RESOURCE_NUMBER_WIN_98		3

#endif // DRIVER_H
