// Driver Entry for sdla driver

#include "stddcls.h"
#include "driver.h"
#include <initguid.h>
#include "guids.h"

NTSTATUS AddDevice(IN PDRIVER_OBJECT DriverObject, IN PDEVICE_OBJECT pdo);
VOID DriverUnload(IN PDRIVER_OBJECT fdo);
NTSTATUS OnRequestComplete(IN PDEVICE_OBJECT fdo, IN PIRP Irp, IN PKEVENT pev);

void FormRegistryPath(PUNICODE_STRING p, PWCH buffer, USHORT cardnumber);

BOOLEAN IsWin98();
BOOLEAN win98 = FALSE;
USHORT devcount = 0, detected_devices_count = 0;

UNICODE_STRING servkey;


/* bit record of available cards */
USHORT available_cards = 0;

WCHAR *fool_proof[] = { L"sdla0",	L"sdla1",	L"sdla2",	L"sdla3",
						L"sdla4",	L"sdla5",	L"sdla6",	L"sdla7",
						L"sdla8",	L"sdla9",	L"sdla10",	L"sdla11",
						L"sdla12",	L"sdla13",	L"sdla14",	L"sdla15" };

/*
number of times any given application registers for use of the
particular card...
if it is zero on registration, a selector and pointer to the card's
base memory is set up (if succeeds); if it falls to zero by
deregistration, the selector is freed
*/
USHORT cardinstances[MAX_CARDS];
UCHAR blockregistration[MAX_CARDS];

//other globals
PDEVICE_EXTENSION dev_array[MAX_CARDS];
PDEVICE_EXTENSION detected_devices_array[MAX_CARDS];
SHORT sdla[MAX_CARDS];

// used for storing log strings
char globalLogString[300];

USHORT DeviceNumber;

RTL_QUERY_REGISTRY_TABLE rtable514[] = {
{ NULL, RTL_QUERY_REGISTRY_DIRECT | RTL_QUERY_REGISTRY_REQUIRED, L"DeviceNumber", (PVOID)&DeviceNumber, 0, NULL, 0 },
{ NULL, 0, NULL, NULL, 0, NULL, 0 }
};

///////////////////////////////////////////////////////////////////////////////

#pragma INITCODE

extern "C" NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject,
								IN PUNICODE_STRING RegistryPath)
{	
#ifdef DEBG
	STRING ansi_string;
	CHAR buffer[125];
	ansi_string.Length = 0;
	ansi_string.MaximumLength = 250;
	ansi_string.Buffer = &buffer[0];
	NTSTATUS status;
#endif
	USHORT i;
	WCHAR tw[80];
	UNICODE_STRING path1;
	
	// DriverEntry
	KdPrint((DRIVERNAME " - Entering DriverEntry: DriverObject %8.8lX\n", DriverObject));
	
	// Insist that OS support at least the WDM level of the DDK we use
	
	if (!IoIsWdmVersionAvailable(1, 0))
	{
		KdPrint((DRIVERNAME " - Expected version of WDM (%d.%2.2d) not available\n", 1, 0));
		return STATUS_UNSUCCESSFUL;
	}
	
	// See if we're running under Win98 or NT:
	
	win98 = IsWin98();
	
#if DBG
	if (win98)
		KdPrint((DRIVERNAME " - Running under Windows 98\n"));
	else
		KdPrint((DRIVERNAME " - Running under NT\n"));
#endif
	
	// Save the name of the service key
	servkey.Buffer = 
		(PWSTR) ExAllocatePool(PagedPool, RegistryPath->Length + sizeof(WCHAR));
	if (!servkey.Buffer)
	{
    	KdPrint((DRIVERNAME " - Unable to allocate %d bytes for copy of service key name\n", RegistryPath->Length + sizeof(WCHAR)));
		return STATUS_INSUFFICIENT_RESOURCES;
	}
	servkey.MaximumLength = RegistryPath->Length + sizeof(WCHAR);
	RtlCopyUnicodeString(&servkey, RegistryPath);

#ifdef DEBG
	//????????????????????????????
	status = RtlUnicodeStringToAnsiString(&ansi_string, &servkey, FALSE);
	if (status != STATUS_SUCCESS)
	{
		DbgPrint("RtlUnicodeStringToAnsiString for registry path failed!\n");
		ExFreePool(servkey.Buffer);
		return STATUS_INSUFFICIENT_RESOURCES;
	}

	DbgPrint("reading registry path...\n");
	for(i=0;i<ansi_string.Length;i++)
	{
		DbgPrint("%C", *(ansi_string.Buffer+i));
	}
	DbgPrint("\nfinished to read registry path.\n");
#endif


	// Initialize function pointers
	DriverObject->DriverUnload = DriverUnload;
	DriverObject->DriverExtension->AddDevice = AddDevice;
	
	DriverObject->MajorFunction[IRP_MJ_CREATE] = DispatchCreate;
	DriverObject->MajorFunction[IRP_MJ_CLOSE] = DispatchClose;
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DispatchControl;
	DriverObject->MajorFunction[IRP_MJ_INTERNAL_DEVICE_CONTROL] = DispatchInternalControl;
	DriverObject->MajorFunction[IRP_MJ_POWER] = DispatchPower;
	DriverObject->MajorFunction[IRP_MJ_PNP] = DispatchPnp;

	
	//???????????????????????
	for(i = 0; i < MAX_CARDS; i++)
	{
		dev_array[i] = NULL;
		sdla[i] = -1;
		cardinstances[i] = 0;
		blockregistration[i] = 0;
	}

	/* no one registered a card yet */
	for (i=0; i < MAX_CARDS; i++)
	{
		FormRegistryPath(&path1, tw, i);
		
		if(RtlQueryRegistryValues(RTL_REGISTRY_SERVICES, path1.Buffer, &rtable514[0], NULL, NULL)
			== STATUS_SUCCESS)
		{
			available_cards |= (0x0001 << i);
			sdla[i] = DeviceNumber;
		}
#ifdef DEBG
		DbgPrint("sdla[%d] : %d\n", i, sdla[i]);
#endif
	}

#ifdef DEBG
	DbgPrint("available_cards : 0x%x\n", available_cards);
#endif

	return STATUS_SUCCESS;
}							// DriverEntry

void FormRegistryPath(PUNICODE_STRING p, PWCH buffer, USHORT cardnumber)
{
	buffer[0] = 0;
	RtlInitUnicodeString(p, buffer);
	p->MaximumLength = 79;
	p->Length = 0;

	RtlAppendUnicodeToString(p, L"SDLA\\Parameters\\");
	RtlAppendUnicodeToString(p, fool_proof[cardnumber]);
}

///////////////////////////////////////////////////////////////////////////////

#pragma PAGEDCODE

VOID DriverUnload(IN PDRIVER_OBJECT DriverObject)
{							// DriverUnload
	PAGED_CODE();
	KdPrint((DRIVERNAME " - Entering DriverUnload: DriverObject %8.8lX\n", DriverObject));
	RtlFreeUnicodeString(&servkey);
}							// DriverUnload

///////////////////////////////////////////////////////////////////////////////

NTSTATUS AddDevice(IN PDRIVER_OBJECT DriverObject, IN PDEVICE_OBJECT pdo)
{							// AddDevice
	PAGED_CODE();
	KdPrint((DRIVERNAME " - Entering AddDevice: DriverObject %8.8lX, pdo %8.8lX\n", DriverObject, pdo));
	
	NTSTATUS status;
	UNICODE_STRING nameString, linkString;
	WCHAR namebuf[64];
	
#ifdef DEBG
	DbgPrint("devcount %d\n", devcount);
#endif

	// Create a functional device object (fdo) to represent the hardware we're managing.
	PDEVICE_OBJECT fdo;

#define xsize sizeof(DEVICE_EXTENSION)
	
	//???????????????????????
	//First device must be "sdla", all others doesn't matter, so we just use
	//devcount to construct the names.

	if(devcount)
	{	_snwprintf(namebuf, arraysize(namebuf), L"\\Device\\sdla_%d", devcount);
		RtlInitUnicodeString(&nameString, namebuf);
	}else{
		RtlInitUnicodeString(&nameString, L"\\Device\\sdla");
	}

	status = IoCreateDevice( DriverObject,
			     xsize,
				 &nameString,
				 FILE_DEVICE_UNKNOWN,
				 0,
				 FALSE,
				 &fdo );

	if (!NT_SUCCESS(status))
	{						// can't create device object
		KdPrint((DRIVERNAME " - IoCreateDevice failed - %X\n", status));
		return status;
	}						// can't create device object
	
	PDEVICE_EXTENSION pdx = (PDEVICE_EXTENSION) fdo->DeviceExtension;

#ifdef DEBG
//	sprintf(pdx->name, "sdla_%d", devcount);
//	DbgPrint("pdx->name : %s\n", pdx->name); 
#endif
	
	fdo->Flags |= DO_BUFFERED_IO;

	if(!devcount)
	{
		RtlInitUnicodeString(&linkString, L"\\DosDevices\\SDLA");	
		status = IoCreateSymbolicLink(&linkString, &nameString);
		
		if (!NT_SUCCESS(status))
		{	
			KdPrint((DRIVERNAME " - IoCreateSymbolicLink failed - %X\n", status));
			IoDeleteDevice(DriverObject->DeviceObject);
			return(status);
		}
	}

	devcount++;
	
	// From this point forward, any error will have side effects that need to
	// be cleaned up. Using a try-finally block allows us to modify the program
	// easily without losing track of the side effects.
	
	__try
	{						// finish initialization
		pdx->DeviceObject = fdo;
		pdx->Pdo = pdo;
		IoInitializeRemoveLock(&pdx->RemoveLock, 0, 0, 0);
		pdx->state = STOPPED;		// device starts in the stopped state
		
		// Link our device object into the stack leading to the PDO
		
		pdx->LowerDeviceObject = IoAttachDeviceToDeviceStack(fdo, pdo);
		if (!pdx->LowerDeviceObject)
		{						// can't attach device
			KdPrint((DRIVERNAME " - IoAttachDeviceToDeviceStack failed\n"));
			status = STATUS_DEVICE_REMOVED;
			__leave;
		}						// can't attach device
		
		// Set power management flags in the device object
		
		fdo->Flags |= DO_POWER_PAGABLE;
		
		// Register a device interface
		
		status = IoRegisterDeviceInterface(pdo, &GUID_INTERFACE_SDLA, NULL, &pdx->ifname);
		if (!NT_SUCCESS(status))
		{						// unable to register interface
			KdPrint((DRIVERNAME " - IoRegisterDeviceInterface failed - %8.8lX\n", status));
			__leave;
		}						// unable to register interface
		
		// Indicate that our initial power state is D0 (fully on). Also indicate that
		// we have a pagable power handler (otherwise, we'll never get idle shutdown
		// messages!)
		
		pdx->syspower = PowerSystemWorking;
		pdx->devpower = PowerDeviceD0;
		POWER_STATE state;
		state.DeviceState = PowerDeviceD0;
		PoSetPowerState(fdo, DevicePowerState, state);
		
		// Clear the "initializing" flag so that we can get IRPs
		
		fdo->Flags &= ~DO_DEVICE_INITIALIZING;
	}						// finish initialization
	__finally
	{						// cleanup side effects
		if (!NT_SUCCESS(status))
		{					// need to cleanup
			if (pdx->ifname.Buffer)
				RtlFreeUnicodeString(&pdx->ifname);
			if (pdx->LowerDeviceObject)
				IoDetachDevice(pdx->LowerDeviceObject);
			IoDeleteDevice(fdo);
		}					// need to cleanup
	}						// cleanup side effects
	
	return status;
	}							// AddDevice
	
	///////////////////////////////////////////////////////////////////////////////
	
#pragma LOCKEDCODE
	
	NTSTATUS CompleteRequest(IN PIRP Irp, IN NTSTATUS status, IN ULONG_PTR info)
	{							// CompleteRequest
		Irp->IoStatus.Status = status;
		Irp->IoStatus.Information = info;
		IoCompleteRequest(Irp, IO_NO_INCREMENT);
		return status;
	}							// CompleteRequest
	
	NTSTATUS CompleteRequest(IN PIRP Irp, IN NTSTATUS status)
	{							// CompleteRequest
		Irp->IoStatus.Status = status;
		IoCompleteRequest(Irp, IO_NO_INCREMENT);
		return status;
	}							// CompleteRequest
	
	///////////////////////////////////////////////////////////////////////////////
	
#pragma PAGEDCODE
	
	NTSTATUS ForwardAndWait(IN PDEVICE_OBJECT fdo, IN PIRP Irp)
	{							// ForwardAndWait
		ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
		PAGED_CODE();
		
		KEVENT event;
		KeInitializeEvent(&event, NotificationEvent, FALSE);
		
		IoCopyCurrentIrpStackLocationToNext(Irp);
		IoSetCompletionRoutine(Irp, (PIO_COMPLETION_ROUTINE) OnRequestComplete,
			(PVOID) &event, TRUE, TRUE, TRUE);
		
		PDEVICE_EXTENSION pdx = (PDEVICE_EXTENSION) fdo->DeviceExtension;
		IoCallDriver(pdx->LowerDeviceObject, Irp);
		KeWaitForSingleObject(&event, Executive, KernelMode, FALSE, NULL);
		return Irp->IoStatus.Status;
	}							// ForwardAndWait
	
	///////////////////////////////////////////////////////////////////////////////
	
#pragma LOCKEDCODE
	
	NTSTATUS OnRequestComplete(IN PDEVICE_OBJECT fdo, IN PIRP Irp, IN PKEVENT pev)
	{							// OnRequestComplete
		KeSetEvent(pev, 0, FALSE);
		return STATUS_MORE_PROCESSING_REQUIRED;
	}							// OnRequestComplete
	
	///////////////////////////////////////////////////////////////////////////////
	
	VOID EnableAllInterfaces(PDEVICE_EXTENSION pdx, BOOLEAN enable)
	{							// EnableAllInterfaces
		IoSetDeviceInterfaceState(&pdx->ifname, enable);
	}							// EnableAllInterfaces
	
	///////////////////////////////////////////////////////////////////////////////
	
	VOID DeregisterAllInterfaces(PDEVICE_EXTENSION pdx)
	{							// DeregisterAllInterfaces
		IoSetDeviceInterfaceState(&pdx->ifname, FALSE);
		RtlFreeUnicodeString(&pdx->ifname);
	}							// DeregisterAllInterfaces
	
	///////////////////////////////////////////////////////////////////////////////
	
#pragma PAGEDCODE
	
	VOID RemoveDevice(IN PDEVICE_OBJECT fdo)
	{							// RemoveDevice
		PAGED_CODE();
		PDEVICE_EXTENSION pdx = (PDEVICE_EXTENSION) fdo->DeviceExtension;
		NTSTATUS status;
		
		
		if (pdx->LowerDeviceObject)
			IoDetachDevice(pdx->LowerDeviceObject);
		
		IoDeleteDevice(fdo);
	}							// RemoveDevice
	
	///////////////////////////////////////////////////////////////////////////////
	
#pragma INITCODE
	
	BOOLEAN IsWin98()
	{							// IsWin98
#ifdef _X86_
		
		// Windows 98 (including 2d ed) supports WDM version 1.0, whereas Win2K
		// supports 1.10.
		
		return !IoIsWdmVersionAvailable(1, 0x10);
#else // not _X86_
		return FALSE;
#endif // not _X86_
	}							// IsWin98
	
	///////////////////////////////////////////////////////////////////////////////
	
#if DBG && defined(_X86_)
#pragma LOCKEDCODE
	
	extern "C" void __declspec(naked) __cdecl _chkesp()
	{
		_asm je okay
			ASSERT(!DRIVERNAME " - Stack pointer mismatch!");
okay:
		_asm ret
	}
	
#endif // DBG
