Attribute VB_Name = "Xapi32mc"
' xapi32mc.bas - this module contains Visul Basic declarations of user functions,
'                structures and constants exported from xapi32mc.dll ( X.25 API
'                for Win32 )
'
' Copyright (c) 1998 Sangoma Technologies Inc.
' Author: Anton Belov
'
' 32-bit, February 1998
'
' More detailed information about usage of C-Style XAPI exports can be obtained
' from xapi32mc.hlp file. Usage of VBasic exports is exactly the same as C exports
' ( VB suffix added to VBasic exports names ). All the VBasic exports declared
' using __stdcall convention, which is exactly what VB needs. All the differences
' between VBasic and C/C++ data types are taken care of inside the .dll, so no
' additional work is required. Use it "as is"
'

' Constants for various return codes from NAPI functions
Public Const SDLAR_FAILURE                  As Integer = 0
Public Const SDLAR_NOT_REGISTERED           As Integer = 0
Public Const SDLAR_CARD_BLOCKED             As Integer = 0
Public Const SDLAR_NOT_PREPARED             As Integer = 0
Public Const SDLAR_SUCCESS                  As Integer = 1
Public Const SDLAR_DEREGISTERED             As Integer = 1
Public Const SDLAR_REGISTERED               As Integer = 1
Public Const SDLAR_ALREADY_LOADED           As Integer = 2
Public Const SDLAR_RELEASED                 As Integer = 2
Public Const SDLAR_ALLOCATED                As Integer = 2
Public Const SDLAR_PREPARED                 As Integer = 2
Public Const SDLAR_ERROR_PREPARING          As Integer = 3

'
' Mailbox ( Control Block ) command constants
'
Public Const READ_CODE_VERSION              As Byte = &H15
Public Const X25_READ                       As Byte = &H22
Public Const X25_WRITE                      As Byte = &H23
Public Const PLACE_CALL                     As Byte = &H30
Public Const ACCEPT_CALL                    As Byte = &H31
Public Const CLEAR_CALL                     As Byte = &H32
Public Const CONFIRM_CLEAR                  As Byte = &H33
Public Const RESET_LOGICAL_CHANNEL          As Byte = &H34
Public Const CONFIRM_RESET                  As Byte = &H35
Public Const RESTART                        As Byte = &H36
Public Const CONFIRM_RESTART                As Byte = &H37
Public Const INTERRUPT                      As Byte = &H38
Public Const CONFIRM_INTERRUPT              As Byte = &H39
Public Const REGISTRATION_REQUEST           As Byte = &H3A
Public Const REGISTRATION_CONFIRMATION      As Byte = &H3B
Public Const IS_DATA_AVAILABLE              As Byte = &H40
Public Const INCOMING_CALL_CONTROL          As Byte = &H41
Public Const CONFIGURE_PVC                  As Byte = &H42
Public Const READ_ACTIVE_CHANNELS           As Byte = &H43
Public Const READ_CHANNEL_CONFIGURATION     As Byte = &H44
Public Const FLUSH_X25_DATA_BUFFERS         As Byte = &H45
Public Const READ_X25_HISTORY_TABLE         As Byte = &H46
Public Const X25_HISTORY_TABLE_CONTROL      As Byte = &H47
Public Const READ_TX_D_BIT_STATUS           As Byte = &H48
Public Const READ_X25_STATISTICS            As Byte = &H49
Public Const FLUSH_X25_STATISTICS           As Byte = &H4A
Public Const READ_HDLC_X25_CONFIGURATION    As Byte = &H50
Public Const SET_HDLC_X25_CONFIGURATION     As Byte = &H51

'
' Some of Control Block return codes
'
Public Const DATA_LENGTH_TOO_BIG            As Byte = &H32
Public Const NO_BUFFS_OR_CLOSED_WIN         As Byte = &H33
Public Const NO_DATA_AVAILABLE              As Byte = &H33
Public Const Z80_TIMEOUT_ERROR              As Byte = &HA

'
' Debugging command trace message (wParam values)
'
Public Const TRACEMSG_CLOSE                 As Long = 0

' Length of control block header
Public Const CB_HEADERLEN                   As Integer = 15

' Maximum Data area length in the control block
' Note that its one less than VC++ declaration since in VB arrays
' declared by index ranges, rather than number of elements
Public Const MDATALEN                       As Integer = 1026

' Constants for Q_D_M_bits of control block
Public Const MASK_QBIT                      As Byte = &H4
Public Const MASK_DBIT                      As Byte = &H2
Public Const MASK_MBIT                      As Byte = &H1

' Control Block structure definition
Type CBlock
    command             As Byte
    buffer_length       As Integer
    return_code         As Byte
    reserved_HDLC       As Byte
    logical_channel     As Integer
    Q_D_M_bits          As Byte
    cause               As Byte
    diagnostic          As Byte
    X25_packet_type     As Byte
    time_stamp(3)       As Byte             ' see note for MDATALEN above
    data(MDATALEN)      As Byte
End Type
        

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' xapi32mc.dll exported functions declarations
'
'

' XAPIAliveVB will always returns 1 - use to check if everything is ok with .dll
Declare Function XAPIAliveVB Lib "xapi32mc.dll" () As Integer

' XAPISetupVB
'
' Initializes xapi32mc.dll and loads SDLA driver, if not loaded already.
' Each application instance MUST call this on initialization.
' Return value :
'   SDLAR_FAILURE        (0) if failed to load SDLA driver
'   SDLAR_SUCCESS        (1) if this is the first setup call and the SDLA driver
'                            loaded successfully
'   SDLAR_ALREADY_LOADED (2) if this is not the first successful setup call
'                            (driver already loaded)
' Note:
'       A trace should be registered before this setup if one will be used at all,
'       but is not necessary.
'
Declare Function XAPISetupVB Lib "xapi32mc.dll" () As Integer

' XAPICleanupVB
'
' Cleans up and unloads SDLA driver if this is the last application instance
' using the driver. You must terminate your trace registration first if one
' was registered, unless you are the last application to use this DLL (but
' you shouldn't assume this).
' Each application instance MUST call this before closing.
' Return value:
'   SDLAR_SUCCESS (1) always
'
Declare Function XAPICleanupVB Lib "xapi32mc.dll" () As Integer

' XAPIqLinkInABMVB
'
' Returns non-zero value if the link is in ABM. 0 otherwise.
'
Declare Function XAPIqLinkInABMVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer

' XAPIControlVB
'
' Emulates BIP interrupt call with control block and a card number as an input.
' Returns  return_code value in the CBlock structure
'
Declare Function XAPIControlVB Lib "xapi32mc.dll" (cb As CBlock, ByVal cardnumber As Byte) As Byte

' XAPIqNotificationVB
'
' Returns non-zero if notify bit is set
'
Declare Function XAPIqNotificationVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Byte

' XAPIqTraceDataAvailableVB
'
' Returns non-zero if trace data is available to be read by a READ_TRACE_DATA call
'
Declare Function XAPIqTraceDataAvailableVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Byte

' XAPIqDataToReadVB
'
' Returns non-zero if there is data to receive, or if the specified
' LC is not in data transfer mode, in which case an X25_READ should be
' issued by the application anyway to look at the return code, which
' would hopefully explain why the link is down
'
Declare Function XAPIqDataToReadVB Lib "xapi32mc.dll" (ByVal logical_channel As Integer, ByVal cardnumber As Byte) As Byte

' XAPIqClearToXmitVB
'
' Returns non-zero if there are available buffers to issue a successful
' X25_WRITE and if the transmit window is open, or if the specified LC
' is not in data transfer mode, in which case an X25_WRITE should be
' issues anyway by the application to look at the return code, which
' would hopefully explain why the link is down
'
Declare Function XAPIqClearToXmitVB Lib "xapi32mc.dll" (ByVal logical_channel As Integer, ByVal cardnumber As Byte) As Byte

' XAPIGetLCTypeVB
'
' Returns a type of given logical channel :
'   0 for PVC
'   1 for Incoming only SVC
'   2 for Two-way SVC
'   3 for Outgoing only SVC
'   255 if logical channel specified is invalid
'
Declare Function XAPIGetLCTypeVB Lib "xapi32mc.dll" (ByVal logical_channel As Integer, ByVal cardnumber As Byte) As Byte

' XAPIGetHighestLCNVB
'
' Returns highest LCN configured, of all SVC's or PVC's
'
Declare Function XAPIGetHighestLCNVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer

' XAPISupervisoryCallVB
'
' Checks the board status bits for the specified information, without
' issuing a mailbox command - therefore works faster than control block commands
'
' Following is a list of infocodes passed as a first parameter to this function :
'
' Returns non-zero if there is data to read on any channel
Public Const XSC_INFO_DATATOREAD            As Byte = &H1
' Returns non-zero if there are buffers available to write a
' packet to the transmit queue (indep. of LC)
Public Const XSC_INFO_BUFFSTOXMIT           As Byte = &H2
' Returns number of packets available to be read
Public Const XSC_LCINFO_NUMPACKETS          As Byte = &H80
' Returns non-zero if the transmit window is open
Public Const XSC_LCINFO_TXWINDOWOPEN        As Byte = &H81
' Returns non-zero if the LC is in data transfer mode
Public Const XSC_LCINFO_IN_DTM              As Byte = &H82
'
'
Declare Function XAPISupervisoryCallVB Lib "xapi32mc.dll" (ByVal infocode As Byte, ByVal logical_channel As Integer, ByVal cardnumber As Byte) As Byte

' XAPIRegisterDebugVB
'
' Returns SDLAR_FAILURE (0) if a debugging command trace is already registered
' or had difficulty creating the window, SDLAR_SUCCESS (1) if successful.
' Input:
'   hWnd        - a window to send notification messages to, and
'   iMessage    - a user-defined message that XAPI will send the application to
'                 signal an XAPI notification. The actual message code is passed
'                 by value to the WPARAM parameter of the window procedure
'
Declare Function XAPIRegisterDebugVB Lib "xapi32mc.dll" (ByVal hInstance As Long, ByVal hWnd As Long, ByVal iMessage As Long) As Byte

' XAPITerminateDebugVB
'
' If a debugging command trace is active and instance matches that passed when
' registered (for security/fault tolerance purposes) then trace is terminated and
' returns 0. If instance doesn't match, returns 1. If no trace is running,
' returns 2.
' The trace is also terminated automatically if the window is closed (see trace
' messages)
'
Declare Function XAPITerminateDebugVB Lib "xapi32mc.dll" (ByVal hInstance As Long) As Byte

' XAPIIsDebugRegisteredVB
'
' Returns True if a debugging command trace is registered and active
' Note: the differences between C and VBasic boolean types and constants are taken
'       care of already
'
Declare Function XAPIIsDebugRegisteredVB Lib "xapi32mc.dll" () As Boolean

' XAPIRegisterCardVB
'
' Registers given card instance
' Return values:
'   SDLAR_NOT_REGISTERED (0)    if failed to obtain a selector (any read or write
'                               calls to the board will result in a general
'                               protection fault)
'   SDLAR_REGISTERED (1)        if succeeded : the card's instance count was
'                               incremented but this is not the first registration
'                               call for this board (it is already in use by an
'                               application)
'   SDLAR_ALLOCATED (2)         if succeeded : the card's instance count was
'                               incremented and this is the first registration call
'                               for this board, and the memory selector was allocated
'
Declare Function XAPIRegisterCardVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer

' XAPIDeregisterCardVB
'
' Returns:
'   SDLAR_NOT_REGISTERED (0)    if card wasn't registered
'   SDLAR_DEREGISTERED (1)      if card's instance count was decremented but isn't
'                               currently zero (i.e. an application is still using
'                               the specified board)
'   SDLAR_RELEASED (2)          if card is now not used by any application, and the
'                               selector was freed
'
Declare Function XAPIDeregisterCardVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer

' XAPIGetCardsVB
'
' Returns a bit record (32-bit) of cards registered for use and cards that exist:
'   [ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
'   For example, if bit R2 is set, then card 2 is registered, if bit E2 is set,
'   then card 2 exists (i.e. a configuration entry exists in registry or WIN.INI)
'
Declare Function XAPIGetCardsVB Lib "xapi32mc.dll" () As Long

'
' Undocumented DLL exports - use at own risk
'
Declare Function XAPIPrepareForLoadingVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer
Declare Function XAPILoadCompleteVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte) As Integer
Declare Function XAPIReadPortByteVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte, ByVal offset As Integer) As Byte
Declare Sub XAPIWritePortByteVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte, ByVal offset As Integer, ByVal data As Byte)
Declare Function XAPIReadByteVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte, ByVal offset As Integer) As Byte
Declare Sub XAPIWriteByteVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte, ByVal offset As Integer, ByVal data As Byte)
Declare Sub XAPIZeroWindowVB Lib "xapi32mc.dll" (ByVal cardnumber As Byte)

