/* x25test.c  */

#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "resource.h"
#include <xapi32mc.h>

HINSTANCE	ghinstance;
HWND		exehwn;

UINT		LCN;				// our logical channel
CBLOCK		cb;

#define X25_READ		0x22
#define X25_WRITE		0x23

char *szClassName = "X25TEST";

/* prototypes */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
void SendFile(HWND hWnd);
int DoSend(HWND hWnd);
void ReceiveFile(HWND hWnd);
int DoReceive(HWND hWnd);

//
// WinMain
//
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG msg;
	BOOL fRetVal;
	int i;
	WNDCLASS wndclass;   // Structure used to register Windows class.

	ghinstance = hInstance;

	if (hPrevInstance != NULL) return(0);

	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = DLGWINDOWEXTRA;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(ghinstance, "x25test");
	wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (!RegisterClass(&wndclass)) {
		return(0);
	}

	exehwn = CreateDialog(ghinstance, "MAINWIN", HWND_DESKTOP, NULL);
	
	i=XAPISetup();
	i=XAPIRegisterCard(0);

	if (!exehwn)  return(0);

	fRetVal = TRUE;
	while (fRetVal)  {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {

			if (msg.message == WM_QUIT)
				fRetVal = FALSE;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	XAPIDeregisterCard(0);
	XAPICleanup();


	return fRetVal;

}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	char s[50];

	switch(iMessage) {
	case WM_INITDIALOG:
		return(1);
	case WM_CLOSE:
		DestroyWindow(hWnd);
		PostQuitMessage(0);
		return(0);

	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDC_QUIT: // QUIT
			PostMessage(hWnd, WM_CLOSE, 0, 0L);
			break;
		case IDC_SETLCN:
			LCN = GetDlgItemInt( hWnd, IDC_LCN, NULL, FALSE );
			sprintf( s, "Logical Channel set to %d", LCN);
			SendDlgItemMessage(exehwn, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
			break;
		case IDC_STARTRCV:
			DoReceive(hWnd);
			break;
		case IDC_STARTSEND:
			DoSend(hWnd);
			break;
		}
		break;

	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			DestroyWindow(hWnd);
			PostQuitMessage(0);
			return(0);
		}
		break;
	}
	return DefDlgProc( hWnd, iMessage, wParam, lParam );
}

// send file with 1000 frames
void SendFile(HWND hWnd)
{
	int i, sBytes;
	char	s[50];

	for(i=0; i<1000;i++) {
		sBytes = DoSend(hWnd);
		if (sBytes==-1) {
			sprintf( s, "Failed to send frame number %d", i );
		} else {
			sprintf( s, "Sent frame number %d - Length = %d", i, sBytes );
		}
		SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
	}
	return;
}

int DoSend(HWND hWnd)
{
	int i;
	BYTE	rCode;
	int		nRecvBytes = 0, loop = 0;
	char	s[50];

	for(i=0; i<=30; i++) {
		cb.command = X25_WRITE;
		if (i==30)
			cb.buffer_length = 20;
		else
			cb.buffer_length = 128;
		cb.logical_channel = LCN;
		if (i==30) 
			cb.Q_D_M_bits = 0; 
		else
			cb.Q_D_M_bits = 1;
		XAPIControl(&cb, 0);
		rCode = cb.return_code;

		if (!rCode) {
			nRecvBytes += cb.buffer_length;
		} else {
			sprintf( s, "X25 error writing data Rcode %02X", rCode);
			SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
			nRecvBytes = -1;
			break;
		}
	}
	sprintf( s, "Sent %d bytes", nRecvBytes );
	SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
	return nRecvBytes;
}


// receive file with 1000 frames
void ReceiveFile(HWND hWnd)
{
	int i, rBytes;
	char	s[50];

	for(i=0; i<1000;i++) {
		rBytes = DoReceive(hWnd);
		if (rBytes==-1) {
			sprintf( s, "Failed to receive frame number %d", i );
		} else {
			sprintf( s, "Received frame number %d - Length = %d", i, rBytes );
		}
		SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
	}
	return;
}

int DoReceive(HWND hWnd)
{
	BOOL	bReadError = FALSE, bMoreData = TRUE;
	BYTE	rCode;
	int		nRecvBytes = 0, loop = 0;
	char	s[50];

	do {
			cb.command = X25_READ;
			cb.buffer_length = 0;
			cb.logical_channel = LCN;
			XAPIControl(&cb, 0);
			rCode = cb.return_code;
			if (!rCode) {
				loop = 0;
				nRecvBytes += cb.buffer_length;
				bMoreData = cb.Q_D_M_bits & 0x01;
			} else {
				switch(rCode) {
				case 0x07:
				case 0x33:
					if (++loop<1000) {
						Sleep(10);
						if ((loop%100) == 0) {
						sprintf( s, "Waiting %d times", loop );
						SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
						}
					} else {
						sprintf( s, "There is no data available at this time");
						SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
						bReadError = TRUE;
					}
					break;
				default:
					sprintf( s, "X25 error reading data");
					SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
					bReadError = TRUE;
					break;
				}
			}
	} while(bMoreData && !bReadError);
	if(bReadError) 
		nRecvBytes = -1;
	sprintf( s, "Received %d bytes", nRecvBytes );
	SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)s );
	SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_SETCURSEL, 
							(WPARAM)SendDlgItemMessage(hWnd, IDC_OUTPUT, LB_GETCOUNT, 0, 0), 0);

	return nRecvBytes;
}

					







