/* x25.h - memory locations and bit masks for X25/SDLA */

/* mailbox usage:
OFFSET	SIZE	CONTENT
0	1	Set to 1 after rest of mailbox set to perform command and
		wait for a reset to 0 to indicate command completion
1	1	command code
2	2	buffer length (size in bytes of data field following mailbox)
4	1	return code
5	1	P/F bit setting
6	2	logical channel number
8	1	Q,D,M bit settings
9	1	cause
10	1	diagnostic
11	1	x.25 packet type
12	4	time stamp
16	var.	data field

All commands except X25_READ and HDLC_READ go to send mailbox.
Mailbox is updated accordingly when command is completed, such as
the return code.
*/


// Memory offsets on S514. Use it for NT and NOT win95
#define	XMO_SENDBOX		0xf6b0	// send mailbox 
#define	XMO_RECVBOX		0xfad0	// receive mailbox
#define	XMO_PVCMAP		0xfef0	// lc type mapping
#define	XMO_ICMAP		0xfef2
#define	XMO_TCMAP		0xfef4
#define	XMO_OCMAP		0xfef6
#define	XMO_TSTAMP		0xfef8	// store current time here
#define	XMO_HDLCSTAT	0xfeff	// hdlc status bits
#define	XMO_XSTAT		0xff00	// x.25 status bits
#define	XMO_LCSTAT		0xff01	// logical channel status bits

/*
// Memory offsets from start of shared memory window. Only on win95.
#define	XMO_SENDBOX		0x16b0	// send mailbox 
#define	XMO_RECVBOX		0x1ad0	// receive mailbox 
#define	XMO_PVCMAP		0x1ef0	// lc type mapping 
#define	XMO_ICMAP		0x1ef2
#define	XMO_TCMAP		0x1ef4
#define	XMO_OCMAP		0x1ef6
#define	XMO_TSTAMP		0x1ef8	// store current time here 
#define	XMO_HDLCSTAT	0x1eff	// hdlc status bits 
#define	XMO_XSTAT		0x1f00	// x.25 status bits 
#define	XMO_LCSTAT		0x1f01	// logical channel status bits 
*/

/* HDLCSTAT BITMASKS */
#define	XBM_CTS_IFRAME		0x80	/* clear to send I frame */
#define	XBM_NUM_I_RECEIVE	0x07	/* number I frames pending read */

/* XSTAT BITMASKS */
#define	XBM_IN_ABM		0x01	/* in ABM */
#define	XBM_X_TOREAD	  	0x02	/* set if x25 data pending read in any LCN */
#define	XBM_TRACE_AVAIL  	0x08	/* trace data available if set */
#define	XBM_NOTIFICATION	0x20	/* set if async/timeout/protocol viol. */
#define	XBM_CTS_X		0x40	/* clear to send an x25 packet */

/* LCSTAT BITMASKS */
#define	XBM_LCN_DTM		0x80	/* logical channel in data transfer mode */
#define	XBM_LCN_TWINOPEN	0x40	/* transmit window open for this lc */
#define	XBM_LCN_NUMTOREAD	0x3f	/* number of packets to be read by X25_READ */

