/*
 * $Header:   C:/anton/apidev/xapi32mc/pvcs/xapi32mc.c_v   1.0   02 Dec 1996 13:50:10   Anton  $
 *
 * $Log:   C:/anton/apidev/xapi32mc/pvcs/xapi32mc.c_v  $
 * 
 *    Rev 1.0   02 Dec 1996 13:50:10   Anton
 * VBASIC stuff added 
 *
 */
/* XAPI32MC.C - X25 driver interface/API provider v1.0		*/
/* 32-bit version targetting VC++ 4.0 DLL					*/
/* Copyright (c) 1994-1996 Sangoma Technologies Inc.		*/
/* Author: Glenn Robins										*/
/* v1.1 - June.96											*/

/*																			
 * Windows 95 Support: "Driver" variable scope is per process, and so all info
 * returned for getcards, registration, and so on, relates to the current
 * process only. This means a loader can actually run while another app is 
 * using the card in Win95. Too bad. It's easier this way.
 */

/* v2.0 -	offsets in x25.h changed to run above S514 driver.
			(SDLA_BLOCKOP *)->offset changed from USHORT to ULONG
			in "sdla.h" to use on S514.
			Note: There is no Win95 support here for S514 PCI card.

*/

/*
	NOTES REGARDING MULTI-THREADING

	There are function groups whose functions must be
	called exclusively within the group as follows:
	(i.e. any function in a particular group must not be
	called simultaneously with another function (or the same
	function) in the same group, which may occur if these
	functions are used in different threads without
	synchronization.  If they are all used within the same
	thread, or of course simultaneously in different processes,
	there is no danger except for calling them in the wrong
	order).

	- 	XAPISetup, XAPICleanup,
		XAPIRegisterDebug, XAPITerminateDebug,
		XAPIIsDebugRegistered

	-	XAPIControl with SET_HDLC_X25_CONFIGURATION
		command combined with ANY other LCN-oriented
		function call

		Note that the above command issued in one
		process will not update LCN-related configuration
		in other processes as it would have done in
		Windows 3.1 for different application instances

	The following functions may be called any time, from
	within any execution thread (or process), assuming they
	are called when appropriate (i.e. don't	call XAPIControl
	on a non-registered card!).

	-	XAPIControl (internally mutex-controlled),
		XAPIGetCards (mutexed with card registration
				functions),
		XAPISupervisoryCall,
		XAPIqNotification,
		XAPIqTraceDataAvailable,
		XAPIqLinkInABM,
		XAPIReadByte, XAPIWriteByte,
		XAPIReadPortByte, XAPIWritePortByte
		XAPIqDataToRead, XAPIqClearToXmit,
		XAPIGetLCType, XAPIGetHighestLCN


*/


#include <windows.h>
#include <windowsx.h>
#include <winioctl.h>
#include <stdio.h>
#include <memory.h>
#include <time.h>

#include "include\sdla.h"	 // SDLA driver stuff. <Development>\WinNT\SDLA
#include "include\win95drv.h"// Win95 wrapper. <Development>\Common\ApiDev\Win95drv

#include "include\xapi32mc.h"				// XAPI functions and constants

#include "include\x25.h"			// memory locations and bit masks for X25/SDLA
#include "include\xapi_m.h"			// mailbox command codes

#define DIAGNOSE
#define COUNT_MAX       500

#ifdef DIAGNOSE
#include "include\xapi_d.h"			// debugging stuff
#endif

#define MakeBCD(v) ((BYTE)( (((BYTE)v/10) << 4) | ((BYTE)v % 10) ))
#define	CAF	10

#define	ADJUST(x)	((x*ca)/CAF)

typedef union {
	WORD lowPVC, hiPVC, lowIC, hiIC, lowTW, hiTW, lowOC, hiOC;
	WORD range[8];
} LCDISTRIB;
LCDISTRIB lc_ranges[MAX_CARDS];
WORD highest_logical_number[MAX_CARDS];

HINSTANCE ghinstance;
HWND dllhwn, b1hwn, sthwn, nchwn, lbhwn;
HANDLE x25drv = INVALID_HANDLE_VALUE;

/* set default timeout to 5 seconds */
DWORD timeout_ticks = 5000;
struct {
	HINSTANCE hInstance;
	HWND hWnd, hNotify;
	UINT iMessage;
	UINT count;
	BOOL registered;
} trace_ID;

BOOL update_timestamp = TRUE;
int num_read_config_retries;

char *szClassName = "XAPIClass";
char *szTitleName = "XAPI Supervisor";

char *xapirc = "   XAPI Return Code:";
char *drvrc = "   SDLA Return Code:";

char *ourinifile = "sdla.ini";
char *iniSection = "XAPI";
char *iniEntryTimeout = "SDLA_timeout_in_seconds";
char *iniEntryTimestamp = "apply_timestamp";
char *iniEntryRetries= "read_config_retries";

DWORD ln;       // dummy variable to hold returned bytes from DevIOCnt

HANDLE xapicontrol_mutex, cardoperation_mutex;

// set to false for Windows NT
BOOLEAN ThisIsWin95;

BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID lpReserved);
int MakeMainWindow(void);
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
char SetWaitOppFlag(ULONG offset, BYTE cardnumber);
char ReadConfiguration(BYTE cardnumber);
unsigned char MapLCNtoPhysical(WORD logical_channel, BYTE cardnumber);
void SayMsg(char far *m);
void SayControlNumber(void);
void AddStringListBox(char far *m);
void ClearTrace(void);
char SayReturnResult(char *s, char rval, char dval, WORD bl);
char InternalDoControl(CBLOCK far *block, BYTE cardnumber);
char ReadConfigNTimes(BYTE cardnumber);

BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID lpReserved)
{
	UINT timeout_secs;
//	OSVERSIONINFO osvi;
	
    switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:

	ghinstance = hDLL;
	dllhwn = NULL;
	trace_ID.registered = FALSE;
	ClearTrace();

	/* read ini settings */
	timeout_secs = (UINT)GetPrivateProfileInt(iniSection, iniEntryTimeout, 10, ourinifile);
	timeout_ticks = (DWORD)timeout_secs * 1000L;
	if (GetPrivateProfileInt(iniSection, iniEntryTimestamp, 1, ourinifile))
		update_timestamp = TRUE;
	else
		update_timestamp = FALSE;
	num_read_config_retries = GetPrivateProfileInt(iniSection, iniEntryRetries, 10, ourinifile);

	// Create and initialize the security descriptor that gains
	// Full access to the mutexes. Then also create SECURITY_ATTRIBUTES
	// structure and pass it to create mutex
	{
		PSECURITY_DESCRIPTOR	pSD;				// sec. descriptor
		SECURITY_ATTRIBUTES		lpMutexAttributes;	// sec. attribs
  
		// Initialize a security descriptor.
 
		pSD = (PSECURITY_DESCRIPTOR) LocalAlloc(LPTR, 
			SECURITY_DESCRIPTOR_MIN_LENGTH);   // defined in WINNT.H 
		if (pSD == NULL) { 
			return FALSE;		// just say not attached
		} 
 
		if (!InitializeSecurityDescriptor(pSD, 
				SECURITY_DESCRIPTOR_REVISION)) { // defined in WINNT.H
			LocalFree((HLOCAL) pSD);
			return FALSE;
		} 
 
		// Add a NULL disc. ACL to the security descriptor.
 
		if (!SetSecurityDescriptorDacl(pSD, 
			TRUE,     /* specifying a disc. ACL  */ 
			(PACL) NULL, 
			FALSE)) { /* not a default disc. ACL */ 
			LocalFree((HLOCAL) pSD);
			return FALSE;
		} 
 
		// Add the security descriptor to the SEC_ATTR structure
		lpMutexAttributes.nLength = sizeof(SECURITY_ATTRIBUTES);
		lpMutexAttributes.lpSecurityDescriptor = pSD;
		lpMutexAttributes.bInheritHandle = FALSE;

		xapicontrol_mutex = CreateMutex(&lpMutexAttributes, FALSE, "NAPI32napicontrolmutex");
		cardoperation_mutex = CreateMutex(&lpMutexAttributes, FALSE, "NAPI32cardoperationmutex");
	}
/*
	// determine if win95 or winnt
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   GetVersionEx(&osvi);
   if (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
		ThisIsWin95 = TRUE;
		win95drv_initialize();
   } else {
		ThisIsWin95 = FALSE;
   }
*/
	//hardcoded to NT/2000 even if running on 98 above PnP driver
	ThisIsWin95 = FALSE;

	break;


	case DLL_PROCESS_DETACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	default:
	    break;
	}
    return TRUE;
}

//!!!!! non-thread-reentrant
void ClearTrace(void)
{
	/* no trace registered */
	trace_ID.hInstance = NULL;
	trace_ID.hWnd = NULL;
	trace_ID.count = 0;
}

/* trace must be registered first! */
//!!!!! non-thread-reentrant
int MakeMainWindow(void)
{
	WNDCLASS wndclass;   // Structure used to register Windows class.
	int title_height, frame_height, frame_width, ca;

	title_height=GetSystemMetrics(SM_CYCAPTION);
	ca = CAF;
	if (title_height > 20)
		ca = ((title_height*CAF) / 20);
//      frame_height=GetSystemMetrics(SM_CYFRAME);
//      frame_width=GetSystemMetrics(SM_CXFRAME);
	frame_height = 1; frame_width = 1;

	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(ghinstance, "superico");
	wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = (HBRUSH)GetStockObject( GRAY_BRUSH );
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (trace_ID.registered == FALSE) {
		if (!RegisterClass(&wndclass)) {
			return(0);
		}
	}
	trace_ID.registered = TRUE;

	trace_ID.hWnd = CreateWindow( szClassName,
				szTitleName,
				WS_OVERLAPPED | WS_MINIMIZE | WS_MINIMIZEBOX | WS_CAPTION | WS_SYSMENU | WS_BORDER,
				CW_USEDEFAULT,
				200,
				ADJUST(402),
				(title_height*15)+(title_height/2),
				HWND_DESKTOP,
				NULL,
				ghinstance,
				NULL);

	if (!trace_ID.hWnd)  return(0);
	b1hwn = CreateWindow("BUTTON", "CLEAR", WS_CHILD | WS_VISIBLE, 2, 2, 100, title_height*2, trace_ID.hWnd, (HMENU)1, trace_ID.hInstance, NULL);
	sthwn = CreateWindow("STATIC", "", WS_CHILD | WS_VISIBLE, 0, title_height*2+4, ADJUST(400),
				 title_height, trace_ID.hWnd, (HMENU)2, trace_ID.hInstance, NULL);
	nchwn = CreateWindow("STATIC", "", WS_CHILD | WS_VISIBLE, 0, title_height*3+4, ADJUST(400),
				 title_height, trace_ID.hWnd, (HMENU)3, trace_ID.hInstance, NULL);
	lbhwn = CreateWindow("LISTBOX", "", WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_DISABLENOSCROLL, 0, title_height*4+(title_height/2), ADJUST(400),
				 title_height*10, trace_ID.hWnd, (HMENU)4, trace_ID.hInstance, NULL);
	ShowWindow(trace_ID.hWnd, SW_MINIMIZE);
	UpdateWindow(trace_ID.hWnd);
	return(1);
}

//!!!!! non-thread-reentrant
void SayMsg(char far *m)
{
	if (trace_ID.hInstance != NULL)
		SendMessage(sthwn, WM_SETTEXT, 0, (LPARAM)m);
}

void AddStringListBox(char far *m)
{
	if (trace_ID.hInstance != NULL) {
		trace_ID.count++;
		if (trace_ID.count == COUNT_MAX)  {
			SendMessage(lbhwn, LB_DELETESTRING, 0, 0L);
			trace_ID.count--;
		}
		SendMessage(lbhwn, LB_ADDSTRING, 0, (LPARAM)m);
		SendMessage(lbhwn, LB_SETCURSEL, trace_ID.count-1, 0L);
	}
}

//!!!!! non-thread-reentrant
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	switch(iMessage) {
	case WM_CLOSE:
		SendMessage(trace_ID.hNotify, trace_ID.iMessage, TRACEMSG_CLOSE, 0L);
		ClearTrace();
		DestroyWindow(hWnd);
		return(0);

	case WM_COMMAND:
		if (GET_WM_COMMAND_HWND(wParam, lParam) == b1hwn) {
			SendMessage(lbhwn, LB_RESETCONTENT, 0, 0L);
			trace_ID.count = 0;
		} else
			return DefWindowProc( hWnd, iMessage, wParam, lParam );
		return(0);
//      case WM_SIZE:
//              if (wParam == SIZE_RESTORED)  return(0);
//              break;
	default:
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	}
}

int SANGOMAAPI XAPIAlive(void)
{
	return(1);
}


//!!!!! non-thread-reentrant
int SANGOMAAPI XAPISetup(void)
{
	if (!ThisIsWin95) {
		if (x25drv != INVALID_HANDLE_VALUE) return(2);
			x25drv = CreateFile("\\\\.\\sdla", GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, (LPSECURITY_ATTRIBUTES)NULL,
				OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, (HANDLE)NULL);
		if (x25drv == INVALID_HANDLE_VALUE) return(0);
	}
	AddStringListBox("Setup complete.");
	return(1);
}

//!!!!! non-thread-reentrant
int SANGOMAAPI XAPICleanup(void)
{
	if (!ThisIsWin95) {
		if (x25drv != INVALID_HANDLE_VALUE) CloseHandle(x25drv);
	}
	if (trace_ID.hWnd)  DestroyWindow(trace_ID.hWnd);
	return(1);
}

LRESULT SANGOMAAPI XAPIGetCards(void)
{
	LRESULT buffer;
	DWORD mresult;

	buffer = 0L;
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult != WAIT_TIMEOUT && mresult != WAIT_FAILED) {
		if (ThisIsWin95)
			buffer = win95drv_getcards();
		else
			DeviceIoControl(x25drv, IOCTL_SDLA_GET_CARDS, (LPVOID)&buffer, sizeof(LRESULT), (LPVOID)&buffer, sizeof(LRESULT), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
		ReleaseMutex(cardoperation_mutex);
	}
	return(buffer);
}

//!!!!! non-thread-reentrant
int SANGOMAAPI XAPIRegisterCard(BYTE cardnumber)
{
	LRESULT rcode;
	DWORD mresult;
	char s[80];
	BYTE rc;
	CARD_TRANSACTION ct;

	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(0);

	if (ThisIsWin95) {
		rcode = win95drv_register(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
		DeviceIoControl(x25drv, IOCTL_SDLA_REGISTER, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
		rcode = ct.rcode;
	}
	if (rcode == 0) {
		ReleaseMutex(cardoperation_mutex);
		return(0);
	}

	sprintf(s, "Card %u registered: return code %lx", cardnumber, rcode);
	AddStringListBox(s);

	if (rcode == SDLAR_ALLOCATED || rcode == SDLAR_REGISTERED) {
		rc = (BYTE)ReadConfigNTimes(cardnumber);
		switch (rc)  {
		case 0x7f:
			AddStringListBox("Config request failed on card registration");
			XAPIDeregisterCard(cardnumber);
			ReleaseMutex(cardoperation_mutex);
			return(0);
		case 0x00:
			break;
		}
		AddStringListBox("SDLA Configuration Read");
	}
	ReleaseMutex(cardoperation_mutex);
	return((int)rcode);
}

/*
//!!!!! non-thread-reentrant
int SANGOMAAPI XAPIRegisterCard(BYTE cardnumber)
{
	LRESULT rcode;
	DWORD mresult;
	char s[80];
	CARD_TRANSACTION ct;

	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(0);

	if (ThisIsWin95) {
		rcode = win95drv_register(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
        DeviceIoControl(x25drv, IOCTL_SDLA_REGISTER, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
		rcode = ct.rcode;
	}
	if (rcode == 0) {
		ReleaseMutex(cardoperation_mutex);
		return(0);
	}

	sprintf(s, "Card %u registered: return code %lx", cardnumber, rcode);
	AddStringListBox(s);

	ReleaseMutex(cardoperation_mutex);
	return((int)rcode);
}
*/


//!!!!! non-thread-reentrant
int SANGOMAAPI XAPIDeregisterCard(BYTE cardnumber)
{
	char s[80];
	CARD_TRANSACTION ct;
	DWORD mresult;

	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(0);

	if (ThisIsWin95) {
		ct.rcode = (BYTE)win95drv_deregister(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
		DeviceIoControl(x25drv, IOCTL_SDLA_DEREGISTER, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
	sprintf(s, "Card %u deregistration call: return code %02x", cardnumber, ct.rcode);
	AddStringListBox(s);
	ReleaseMutex(cardoperation_mutex);
	return((int)ct.rcode);
}

int SANGOMAAPI XAPIPrepareForLoading(BYTE cardnumber)
{
	char s[80];
	CARD_TRANSACTION ct;
	DWORD mresult;

	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(3);

	if (ThisIsWin95) {
		ct.rcode = (BYTE)win95drv_preparetoload(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
		DeviceIoControl(x25drv, IOCTL_SDLA_PREPARE_TO_LOAD, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
	sprintf(s, "Card %u load preparation call: return code %02x", cardnumber, ct.rcode);
	AddStringListBox(s);
	ReleaseMutex(cardoperation_mutex);
	return((int)ct.rcode);
}

int SANGOMAAPI XAPILoadComplete(BYTE cardnumber)
{
	char s[80];
	CARD_TRANSACTION ct;
	DWORD mresult;

	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(0);

	if (ThisIsWin95) {
		ct.rcode = (BYTE)win95drv_loadcomplete(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
		DeviceIoControl(x25drv, IOCTL_SDLA_LOAD_COMPLETE, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
	sprintf(s, "Card %u load complete call: return code %02x", cardnumber, ct.rcode);
	AddStringListBox(s);
	ReleaseMutex(cardoperation_mutex);
	return((int)ct.rcode);
}

unsigned char SANGOMAAPI XAPIReadPortByte(BYTE cardnumber, WORD offset)
{
	XFER_BYTE_CONTROL xbc;
	if (ThisIsWin95) {
		return(win95drv_readportbyte(cardnumber, offset));
	}
	xbc.cardnumber = cardnumber;
	xbc.offset = offset;
	DeviceIoControl(x25drv, IOCTL_SDLA_READ_PORT_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	return((BYTE)xbc.value);
}

void SANGOMAAPI XAPIWritePortByte(BYTE cardnumber, WORD offset, BYTE data)
{
	XFER_BYTE_CONTROL xbc;
	if (ThisIsWin95) {
		win95drv_writeportbyte(cardnumber, offset, data);
	} else {
		xbc.cardnumber = cardnumber;
		xbc.offset = offset;
		xbc.value = data;
		DeviceIoControl(x25drv, IOCTL_SDLA_WRITE_PORT_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
}

unsigned char SANGOMAAPI XAPIReadByte(BYTE cardnumber, ULONG offset)
{
	XFER_BYTE_CONTROL xbc;
	if (ThisIsWin95) {
		return(win95drv_readbyte(cardnumber, (USHORT)offset));
	}
	xbc.cardnumber = cardnumber;
	xbc.offset = offset;
	DeviceIoControl(x25drv, IOCTL_SDLA_READ_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	return((BYTE)xbc.value);
}

void SANGOMAAPI XAPIWriteByte(BYTE cardnumber, ULONG offset, BYTE data)
{
	XFER_BYTE_CONTROL xbc;
	if (ThisIsWin95) {
		win95drv_writebyte(cardnumber, (USHORT)offset, data);
	} else {
		xbc.cardnumber = cardnumber;
		xbc.offset = offset;
		xbc.value = data;
		DeviceIoControl(x25drv, IOCTL_SDLA_WRITE_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
}
//----------------------------------------

void SANGOMAAPI XAPIStartS514(BYTE cardnumber)//, WORD offset, BYTE data)
{
	XFER_BYTE_CONTROL xbc;
//	if (ThisIsWin95) {
//		win95drv_writebyte(cardnumber);//, offset, data);
//	} else {
		xbc.cardnumber = cardnumber;
		xbc.offset = 0;//offset;
		xbc.value = 0;//data;
		DeviceIoControl(x25drv, IOCTL_START_S514, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
//	}
}

void SANGOMAAPI XAPIStopS514(BYTE cardnumber)//, WORD offset, BYTE data)
{
	XFER_BYTE_CONTROL xbc;
//	if (ThisIsWin95) {
//		win95drv_writebyte(cardnumber);//, offset, data);
//	} else {
		xbc.cardnumber = cardnumber;
		xbc.offset = 0;//offset;
		xbc.value = 0;//data;
		DeviceIoControl(x25drv, IOCTL_STOP_S514, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
//	}
}

//--------------------------------------------

void SANGOMAAPI XAPIZeroWindow(BYTE cardnumber)
{
	CARD_TRANSACTION ct;
	if (ThisIsWin95) {
		win95drv_zerowindow(cardnumber);
	} else {
		ct.cardnumber = cardnumber;
        DeviceIoControl(x25drv, IOCTL_SDLA_ZERO_WINDOW, (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
}

//!!!!! non-thread-reentrant
char SANGOMAAPI XAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage)
{
	if (trace_ID.hInstance == NULL) {
		trace_ID.hInstance = hInstance;
		trace_ID.hNotify = hWnd;
		trace_ID.iMessage = iMessage;
		trace_ID.count = 0;
		if (!MakeMainWindow())  return(0);
		AddStringListBox("Trace Initiated...");
		return(1);
	}
	return(0);
}

//!!!!! non-thread-reentrant (iffy)
BOOL SANGOMAAPI XAPIIsDebugRegistered(void)
{
	return ((trace_ID.hInstance == NULL) ? FALSE : TRUE);
}

//!!!!! non-thread-reentrant
char SANGOMAAPI XAPITerminateDebug(HINSTANCE hInstance)
{
	if (trace_ID.hInstance == NULL)  return(2);
	if (trace_ID.hInstance != hInstance)  return(1);
	DestroyWindow(trace_ID.hWnd);
	ClearTrace();
	return(0);
}


int SANGOMAAPI XAPIqLinkInABM(BYTE cardnumber)
{
	XFER_BYTE_CONTROL xbc;
	xbc.cardnumber = cardnumber;
	xbc.offset = XMO_XSTAT;
	if (ThisIsWin95) {
		xbc.value = win95drv_readbyte(cardnumber, (USHORT)xbc.offset);
	} else {
		DeviceIoControl(x25drv, IOCTL_SDLA_READ_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
	return (xbc.value & XBM_IN_ABM);
}

//!!!!! non-thread-reentrant (iffy)
void SayControlNumber(void)
{
	char s[80];
	static unsigned long n = 0;
	n++;
	if (trace_ID.hInstance != NULL) {
		sprintf(s, "XAPIControl calls: %lu", n);
		SendMessage(nchwn, WM_SETTEXT, 0, (LPARAM)s);
	}
}

char SANGOMAAPI XAPIControl(CBLOCK far *block, BYTE cardnumber)
{
	char rcode;
	rcode = InternalDoControl(block, cardnumber);
	if (block->command == SET_HDLC_X25_CONFIGURATION) {
		ReadConfigNTimes(cardnumber);
	}
	return(rcode);
}

char InternalDoControl(CBLOCK far *block, BYTE cardnumber)
{
	char s[80];
	SDLA_BLOCKOP memblock;
    struct tm *newtime;
    time_t long_time;
	BYTE stamp[5];
	char flag;
	DWORD mresult;

#ifdef DIAGNOSE
if (block->command <= HIGHEST_COMM) {
	sprintf(s, "LCN%4d  Card%2d  0x%02x %s", block->logical_channel, cardnumber, block->command, diagnose_command[block->command]);
	SayMsg(s);
	AddStringListBox(s);
}
else    AddStringListBox("UNKNOWN COMMAND!");
#endif

	memblock.card_number = cardnumber;

	SayControlNumber();

	// start system-wide critical section
	// release mutex in SayReturnResult!!!
	mresult = WaitForSingleObject(xapicontrol_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		block->return_code = Z80_TIMEOUT_ERROR;
		return(SayReturnResult(drvrc, block->return_code, block->data[0], block->buffer_length));
	}

	if (update_timestamp) {
		/* set the timestamp on the board */
        time( &long_time );                /* Get time as long integer. */
        newtime = localtime( &long_time ); /* Convert to local time. */

		stamp[0] = MakeBCD(newtime->tm_mon);
		stamp[1] = MakeBCD(newtime->tm_mday);
		stamp[2] = MakeBCD(newtime->tm_hour);
		stamp[3] = MakeBCD(newtime->tm_min);
		stamp[4] = MakeBCD(newtime->tm_sec);

	// might want to declare and set this block once on init instead to speed things up a bit
		memblock.offset = XMO_TSTAMP;
		memblock.num_bytes = 5;
		memblock.buffer = stamp;
		if (ThisIsWin95)
			win95drv_writeblock(cardnumber, (USHORT)memblock.offset, memblock.num_bytes, memblock.buffer);
		else
			DeviceIoControl(x25drv, IOCTL_SDLA_WRITE_BLOCK, (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}

	/* decide on correct mailbox to use according to command */
	if (block->command == X25_READ)  memblock.offset = XMO_RECVBOX+1;
	else  memblock.offset = XMO_SENDBOX+1;

	/* reject block if buffer_length is too high, no matter what */
	if (block->buffer_length > MDATALEN)
	{
		return(SayReturnResult(	xapirc, block->return_code=DATA_LENGTH_TOO_BIG, 
								block->data[0], block->buffer_length));
	}

	switch(block->command) {
	case X25_WRITE:
		if (!XAPIqClearToXmit(block->logical_channel, cardnumber))
			return(SayReturnResult(xapirc, block->return_code=NO_BUFFS_OR_CLOSED_WIN, block->data[0], block->buffer_length));
		break;
	case X25_READ:
		if (!XAPIqDataToRead(block->logical_channel, cardnumber))
			return(SayReturnResult(xapirc, block->return_code=NO_DATA_AVAILABLE, block->data[0], block->buffer_length));
		break;
	}

	/* submit to appropriate mailbox via driver */
	memblock.num_bytes = block->buffer_length + CB_HEADERLEN;
	memblock.buffer = block;
	if (ThisIsWin95)
		win95drv_writeblock(cardnumber, (USHORT)memblock.offset, memblock.num_bytes, memblock.buffer);
	else
		DeviceIoControl(x25drv, IOCTL_SDLA_WRITE_BLOCK, (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);

	/* submitted to mailbox... now set flag and wait to clear */
	flag = SetWaitOppFlag(memblock.offset-1, cardnumber);

	if (!flag) block->return_code = Z80_TIMEOUT_ERROR;
	else {
		/* give application a copy of the mailbox after the transaction */
		memblock.num_bytes = CB_HEADERLEN;
		if (ThisIsWin95)
			win95drv_readblock(cardnumber, (USHORT)memblock.offset, memblock.num_bytes, memblock.buffer);
		else
			DeviceIoControl(x25drv, IOCTL_SDLA_READ_BLOCK, (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
		memblock.num_bytes = block->buffer_length;
		memblock.buffer = block->data;
		memblock.offset += CB_HEADERLEN;
		if (ThisIsWin95)
			win95drv_readblock(cardnumber, (USHORT)memblock.offset, memblock.num_bytes, memblock.buffer);
		else
			DeviceIoControl(x25drv, IOCTL_SDLA_READ_BLOCK, (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPVOID)&memblock, sizeof(SDLA_BLOCKOP), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	}
	return(SayReturnResult(drvrc, block->return_code, block->data[0], block->buffer_length));
}

/* 's' must be of size less than (80-5) bytes incl. null */
char SayReturnResult(char *s, char rval, char dval, WORD bl)
{
	char a[80];
	if (bl > 0)
		sprintf(a, "%s 0x%02x  Data[0] = 0x%02x", s, (BYTE)rval, (BYTE)dval);
	else
		sprintf(a, "%s 0x%02x", s, (BYTE)rval);
	AddStringListBox(a);

	// someone else can do a control call now
	ReleaseMutex(xapicontrol_mutex);
	return(rval);
}

// runs inside xapicontrol_mutex
char SetWaitOppFlag(ULONG offset, BYTE cardnumber)
/* returns 0 if timed-out, 1 if ok */
{
	DWORD t1;
	char s[100];
	XFER_BYTE_CONTROL xbc;

	t1 = GetTickCount();
	/* set opp_flag to 1 */
	xbc.cardnumber = cardnumber;
	xbc.offset = offset;
	xbc.value = 0x01;
	if (ThisIsWin95)
		win95drv_writebyte(cardnumber, (USHORT)xbc.offset, xbc.value);
	else
		DeviceIoControl(x25drv, IOCTL_SDLA_WRITE_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);

	/* loop forever until 0 */
	do {
		if (GetTickCount() > (t1+timeout_ticks)) {
			if (trace_ID.hInstance != NULL) {
				sprintf(s, "Card %u is not responding!  This may cause instability in calling applications.", HIWORD(offset));
				MessageBox(HWND_DESKTOP, s, "XAPI Hardware Timeout Error", MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL);
			}
			return(0);
		}
		if (ThisIsWin95)
			xbc.value = win95drv_readbyte(cardnumber, (USHORT)xbc.offset);
		else
			DeviceIoControl(x25drv, IOCTL_SDLA_READ_BYTE, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	} while (xbc.value);
	return(1);
}

char SANGOMAAPI XAPIqClearToXmit(WORD logical_channel, BYTE cardnumber)
/* logic:
first test LCN in xfer mode: no -> passthru (return code 1)
logical and of transmit window open and buffers available: yes -> passthru
else return 0
*/
{
	BYTE x, pc;
	pc = MapLCNtoPhysical(logical_channel, cardnumber);
	x = XAPIReadByte(cardnumber, XMO_XSTAT+(WORD)pc);
	if (!(x & XBM_LCN_DTM))  return(1);
	if (x & XBM_LCN_TWINOPEN)  {
		x = XAPIReadByte(cardnumber, XMO_XSTAT);
		if (x & XBM_CTS_X)  return(1);
	}
	return (0);
}

char SANGOMAAPI XAPIqDataToRead(WORD logical_channel, BYTE cardnumber)
/* logic:
first test LCN in xfer mode: no -> passthru (return code 1)
if there's something to receive -> passthru
if there's an async packet -> passthru
else return 0
*/
{
	BYTE x, pc;
	pc = MapLCNtoPhysical(logical_channel, cardnumber);
	x = XAPIReadByte(cardnumber, XMO_XSTAT+(WORD)pc);
	if (!(x & XBM_LCN_DTM))  return(1);
	if (x & XBM_LCN_NUMTOREAD)  return(1);
	if (XAPIqNotification(cardnumber))  return(1);
	return (0);
}


unsigned char MapLCNtoPhysical(WORD logical_channel, BYTE cardnumber)
/* translate logical channel number to physical offset in LC table */
/* returns 0 if invalid channel number */
{
	BYTE type, low, hi;
	WORD subt;

	/* check if only need simple mapping first */
	if (highest_logical_number[cardnumber] < 0x100)  return((unsigned char)logical_channel);

	type = XAPIGetLCType(logical_channel, cardnumber);
	if (type == 0x7f)  return(0);
	low = XAPIReadByte(cardnumber, XMO_PVCMAP+(type*2));
	hi = XAPIReadByte(cardnumber, XMO_PVCMAP+(type*2)+1);
	subt = ((WORD)hi * 256) + (WORD)low;
	return((unsigned char)(logical_channel - subt));
}

unsigned char SANGOMAAPI XAPIGetLCType(WORD lc, BYTE cardnumber)
/* returns 0 if in PVC range, 1 in IC, 2 in TW, 3 in OC, 0x7f if invalid */
{
	int c;
	for (c=0; c < 4; c++)  {
		if (lc >= lc_ranges[cardnumber].range[2*c] && lc <= lc_ranges[cardnumber].range[2*c+1])
			return(c);
	}
	return(0x7f);
}

char SANGOMAAPI XAPIqNotification(BYTE cardnumber)
{
	BYTE x;
	x = XAPIReadByte(cardnumber, XMO_XSTAT);
	return (x & XBM_NOTIFICATION);
}

char SANGOMAAPI XAPIqTraceDataAvailable(BYTE cardnumber)
{
	BYTE x;
	x = XAPIReadByte(cardnumber, XMO_XSTAT);
	return (x & XBM_TRACE_AVAIL);
}

char ReadConfigNTimes(BYTE cardnumber)
/* returns 0x7f if failed after num_read_config_retries, 0 if ok */
/* if ReadConfiguration gives a TIMEOUT error, forget retries */
{
	int i;
	BYTE rc;

	i=0;
	do 
	{
		i++;
		rc = (BYTE)ReadConfiguration(cardnumber);
		if (rc == Z80_TIMEOUT_ERROR) return(0x7f);
	} while (rc && i <= num_read_config_retries);
	if (i > num_read_config_retries)
		return(0x7f);
	return(0);
}

//!!!!! non-thread-reentrant including all tests involving
//  lc_ranges array
char ReadConfiguration(BYTE cardnumber)
/* returns rcode (non-zero) if couldn't read it, return_code (0) if did */
{
	CBLOCK cb;
	int c;
	WORD largest, n;
	unsigned char rcode;

	cb.command = READ_CHANNEL_CONFIGURATION;
	cb.buffer_length = 0;
	cb.logical_channel = 0;
	cb.return_code = 0x7f;
	if ((rcode=InternalDoControl(&cb, cardnumber)) != 0)  return(rcode);
	/* let's keep a local copy */
	_fmemcpy((void far *)&lc_ranges[cardnumber], cb.data, 16);

	/* find highest logical channel number to decide if need
	complex or simple mapping */
	largest = 0;
	for (c=0; c < 4; c++)  {
		if ((n=lc_ranges[cardnumber].range[2*c+1]) > largest)
			largest = n;
	}
	highest_logical_number[cardnumber] = largest;
	return(rcode);
}

WORD SANGOMAAPI XAPIGetHighestLCN(BYTE cardnumber)
{
	return(highest_logical_number[cardnumber]);
}


char SANGOMAAPI XAPISupervisoryCall(BYTE infocode, WORD lcn, BYTE cardnumber)
/* assumes valid lcn (check valid by call to XAPIGetLCType) */
{
	BYTE lstat, pc;

	if (lcn > 0 && (infocode & XSC_INFO_PC)) {
		pc = MapLCNtoPhysical(lcn, cardnumber);
		lstat = XAPIReadByte(cardnumber, XMO_XSTAT+(WORD)pc);
		switch(infocode) {
		case XSC_LCINFO_NUMPACKETS:
			return(lstat & XBM_LCN_NUMTOREAD);
		case XSC_LCINFO_TXWINDOWOPEN:
			return(lstat & XBM_LCN_TWINOPEN);
		case XSC_LCINFO_IN_DTM:
			return(lstat & XBM_LCN_DTM);
		}
		return(0);
	}
	lstat = XAPIReadByte(cardnumber, XMO_XSTAT);
	switch(infocode) {
	case XSC_INFO_DATATOREAD:
		return(lstat & XBM_X_TOREAD);
	case XSC_INFO_BUFFSTOXMIT:
		return(lstat & XBM_CTS_X);
	}
	return(0);
}


/*---------------------------------------------------------------------------------------
 * Ok, here start function implementation to be used by VBasic developers. Most of them 
 * are just dummy __stdcall calls to regular XAPI.. functions, however when passing CBLOCK
 * structure - we have to take care of the fact that all structures in VB are 4-byte 
 * aligned - so we have to do some transformations with it
 *
 */

/* VBasic control block - same as CBLOCK, but packed on 4 bytes boundaries */
#pragma pack(4)
typedef struct {
	BYTE command;
	WORD buffer_length;
	BYTE return_code;
	BYTE reserved_HDLC;
	WORD logical_channel;
	BYTE Q_D_M_bits;
	BYTE cause;
	BYTE diagnostic;
	BYTE X25_packet_type;
	BYTE time_stamp[4];
	BYTE data[MDATALEN];
} CBLOCK_VB;

SANGOMAIMPORT
SHORT __stdcall XAPIAliveVB(void) 
{
	return (SHORT)XAPIAlive();
}

SANGOMAIMPORT
SHORT __stdcall XAPISetupVB(void)
{
	return (SHORT)XAPISetup();
}

SANGOMAIMPORT
SHORT  __stdcall XAPICleanupVB(void)
{
	return (SHORT)XAPICleanup();
}

SANGOMAIMPORT
SHORT __stdcall XAPIqLinkInABMVB(BYTE cardnumber)
{
	return (SHORT)XAPIqLinkInABM(cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPIControlVB(CBLOCK_VB *cb_vb, BYTE cardnumber)
{
	CBLOCK	cb;
	char	rval;
	
	// translate CBLOCK_VB to CBLOCK
	cb.command				= cb_vb->command;
	cb.buffer_length		= cb_vb->buffer_length;
	cb.return_code			= cb_vb->return_code;
	cb.reserved_HDLC		= cb_vb->reserved_HDLC;
	cb.logical_channel		= cb_vb->logical_channel;
	cb.Q_D_M_bits			= cb_vb->Q_D_M_bits;
	cb.cause				= cb_vb->cause;
	cb.diagnostic			= cb_vb->diagnostic;
	cb.X25_packet_type		= cb_vb->X25_packet_type;
	*(ULONG*)(cb.time_stamp)= *(ULONG *)(cb_vb->time_stamp);
	memcpy( cb.data, cb_vb->data, cb_vb->buffer_length ); // just copy what we need

	rval = XAPIControl( &cb, cardnumber );

	// copy stuff back
	cb_vb->command					= cb.command;
	cb_vb->buffer_length			= cb.buffer_length;
	cb_vb->return_code				= cb.return_code;
	cb_vb->reserved_HDLC			= cb.reserved_HDLC;
	cb_vb->logical_channel			= cb.logical_channel;
	cb_vb->Q_D_M_bits				= cb.Q_D_M_bits;
	cb_vb->cause					= cb.cause; 
	cb_vb->diagnostic				= cb.diagnostic;
	cb_vb->X25_packet_type			= cb.X25_packet_type;
	*(ULONG *)(cb_vb->time_stamp)	= *(ULONG*)(cb.time_stamp);
	memcpy( cb_vb->data, cb.data, cb.buffer_length );	// copy back just what we need from data area

	return rval;
}

SANGOMAIMPORT
char __stdcall XAPIqNotificationVB(BYTE cardnumber)
{
	return XAPIqNotification(cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPIqTraceDataAvailableVB(BYTE cardnumber)
{
	return XAPIqTraceDataAvailable(cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPIqDataToReadVB(WORD logical_channel, BYTE cardnumber)
{
	return XAPIqDataToRead(logical_channel, cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPIqClearToXmitVB(WORD logical_channel, BYTE cardnumber)
{
	return XAPIqClearToXmit(logical_channel, cardnumber);
}

SANGOMAIMPORT
unsigned char __stdcall XAPIGetLCTypeVB(WORD logical_channel, BYTE cardnumber)
{
	return XAPIGetLCType(logical_channel, cardnumber);
}

SANGOMAIMPORT
WORD __stdcall XAPIGetHighestLCNVB(BYTE cardnumber)
{
	return XAPIGetHighestLCN(cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPISupervisoryCallVB(BYTE infocode, WORD logical_channel, BYTE cardnumber)
{
	return XAPISupervisoryCall(infocode, logical_channel, cardnumber);
}

SANGOMAIMPORT
char __stdcall XAPIRegisterDebugVB(HINSTANCE hInstance, HWND hWnd, UINT iMessage)
{
	return XAPIRegisterDebug(hInstance, hWnd, iMessage);
}

SANGOMAIMPORT
char __stdcall XAPITerminateDebugVB(HINSTANCE hInstance)
{
	return XAPITerminateDebug(hInstance);
}

// Here we have to take care of differences between C and VBasic : 
// In C       BOOL is 32 bits, TRUE is 1, FALSE is 0. 
// In Vbasic  Boolean in 16 bits, True is 0xffff, False is 0
SANGOMAIMPORT
SHORT __stdcall XAPIIsDebugRegisteredVB(void)
{
	return XAPIIsDebugRegistered() ? (SHORT)0xffff : 0;
}

SANGOMAIMPORT
SHORT __stdcall XAPIRegisterCardVB(BYTE cardnumber)
{
	return (SHORT)XAPIRegisterCard(cardnumber);
}

SANGOMAIMPORT
SHORT __stdcall XAPIDeregisterCardVB(BYTE cardnumber)
{
	return (SHORT)XAPIDeregisterCard(cardnumber);
}

SANGOMAIMPORT
LONG __stdcall XAPIGetCardsVB(void)
{
	return (LONG)XAPIGetCards();
}

SANGOMAIMPORT
SHORT __stdcall XAPIPrepareForLoadingVB(BYTE cardnumber)
{
	return (SHORT)XAPIPrepareForLoading(cardnumber);
}

SANGOMAIMPORT
SHORT __stdcall XAPILoadCompleteVB(BYTE cardnumber)
{
	return (SHORT)XAPILoadComplete(cardnumber);
}

SANGOMAIMPORT
unsigned char __stdcall XAPIReadPortByteVB(BYTE cardnumber, WORD offset)
{
	return XAPIReadPortByte(cardnumber, offset);
}

SANGOMAIMPORT
void __stdcall XAPIWritePortByteVB(BYTE cardnumber, WORD offset, BYTE data)
{
	XAPIWritePortByte(cardnumber, offset, data);
}

SANGOMAIMPORT 
unsigned char __stdcall XAPIReadByteVB(BYTE cardnumber, WORD offset)
{
	return XAPIReadByte(cardnumber, offset);
}

SANGOMAIMPORT 
void __stdcall XAPIWriteByteVB(BYTE cardnumber, WORD offset, BYTE data)
{
	XAPIWriteByte(cardnumber, offset, data);
}

SANGOMAIMPORT
void __stdcall XAPIZeroWindowVB(BYTE cardnumber)
{
	XAPIZeroWindow(cardnumber);
}
